<?php

namespace Boxberry\Api\Entity;

use Bitrix\Main\Localization\Loc;
use Boxberry\Api\Exceptions\BoxberryApiException;
Loc::loadMessages(__FILE__);
class DeliveryCalculationResult
{
    private int $totalPricePickup = 0;
    private int $totalPriceCourier = 0;
    private int $priceBasePickup = 0;
    private int $priceServicePickup = 0;
    private int $priceBaseCourier = 0;
    private int $deliveryPeriodPickup = 0;
    private int $deliveryPeriodCourier = 0;
    private int $priceServiceCourier = 0;

    /**
     * @throws BoxberryApiException
     */
    public function __construct($data = null)
    {
        if (empty($data)) {
            throw new BoxberryApiException(Loc::getMessage('EMPTY_DELIVERY_CALCULATION_RESULT'));
        }

        if (isset($data['err'])) {
            throw new BoxberryApiException($data['err']);
        }

        if (isset($data['error'], $data['message']) && $data['error'] === true) {
            throw new BoxberryApiException($data['message']);
        }

        if (!isset($data['result']['DeliveryCosts']) || !is_array($data['result']['DeliveryCosts'])) {
            throw new BoxberryApiException(Loc::getMessage('DELIVERY_TO_SELECTED_PLACE_IS_NOT_AVAILABLE'));
        }

        foreach ($data['result']['DeliveryCosts'] as $deliveryCost) {
            if ($deliveryCost['DeliveryTypeId'] === DeliveryCalculationParams::PICKUP_DELIVERY_TYPE_ID) {
                $this->priceServicePickup = $deliveryCost['PriceService'];
                $this->totalPricePickup = $deliveryCost['TotalPrice'];
                $this->priceBasePickup = $deliveryCost['PriceBase'];
                $this->deliveryPeriodPickup = $deliveryCost['DeliveryPeriod'];
            }

            if ($deliveryCost['DeliveryTypeId'] === DeliveryCalculationParams::COURIER_DELIVERY_TYPE_ID) {
                $this->priceServiceCourier = $deliveryCost['PriceService'];
                $this->totalPriceCourier = $deliveryCost['TotalPrice'];
                $this->priceBaseCourier = $deliveryCost['PriceBase'];
                $this->deliveryPeriodCourier = $deliveryCost['DeliveryPeriod'];
            }
        }
    }

    public function getTotalPricePickup(): int
    {
        return $this->totalPricePickup;
    }

    public function setTotalPricePickup(int $totalPricePickup): void
    {
        $this->totalPricePickup = $totalPricePickup;
    }

    public function getTotalPriceCourier(): int
    {
        return $this->totalPriceCourier;
    }

    public function setTotalPriceCourier(int $totalPriceCourier): void
    {
        $this->totalPriceCourier = $totalPriceCourier;
    }

    public function getPriceBasePickup(): int
    {
        return $this->priceBasePickup;
    }

    public function setPriceBasePickup(int $priceBasePickup): void
    {
        $this->priceBasePickup = $priceBasePickup;
    }

    public function getPriceBaseCourier(): int
    {
        return $this->priceBaseCourier;
    }

    public function setPriceBaseCourier(int $priceBaseCourier): void
    {
        $this->priceBaseCourier = $priceBaseCourier;
    }

    public function getDeliveryPeriodPickup(): int
    {
        return $this->deliveryPeriodPickup;
    }

    public function setDeliveryPeriodPickup(int $deliveryPeriodPickup): void
    {
        $this->deliveryPeriodPickup = $deliveryPeriodPickup;
    }

    public function getDeliveryPeriodCourier(): int
    {
        return $this->deliveryPeriodCourier;
    }

    public function setDeliveryPeriodCourier(int $deliveryPeriodCourier): void
    {
        $this->deliveryPeriodCourier = $deliveryPeriodCourier;
    }

    public function getPriceServicePickup(): int
    {
        return $this->priceServicePickup;
    }

    public function setPriceServicePickup(int $priceServicePickup): void
    {
        $this->priceServicePickup = $priceServicePickup;
    }

    public function getPriceServiceCourier(): int
    {
        return $this->priceServiceCourier;
    }

    public function setPriceServiceCourier(int $priceServiceCourier): void
    {
        $this->priceServiceCourier = $priceServiceCourier;
    }


}