<?php

namespace Boxberry\Api\Entity;

use Boxberry\Api\Exceptions\BoxberryApiException;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class ParcelInfoResult
{
    private array $items = [];
    private array $weights = [];
    private string $fio = '';
    private string $phone = '';
    private ?string $email = '';
    private int $deliveryType = 0;
    private string $deliveryCost = '';
    private string $declaredCost = '';
    private string $paymentAmount = '';
    private ?string $postCode = '';
    private string $city = '';
    private ?string $addressRecipient = '';
    private ?string $pointCode = '';
    private ?string $pointAddress = '';
    private int $issueType = 0;


    /**
     * @throws BoxberryApiException
     */
    public function __construct($data = [])
    {
        if (empty($data)) {
            throw new BoxberryApiException(Loc::getMessage('PARCEL_INFO_RESULT_EMPTY_DATA'));
        }

        if (isset($data['err'])) {
            throw new BoxberryApiException($data['err']);
        }

        $this->items = $data[0]['products'];
        $this->weights = $data[0]['boxes_data'];
        $this->fio = $data[0]['recipient_data']['full_name'];
        $this->phone = $data[0]['recipient_data']['phone'];
        $this->email = $data[0]['recipient_data']['email'];
        $this->deliveryType = $data[0]['delivery_type'];
        $this->deliveryCost = $data[0]['delivery_cost'];
        $this->declaredCost = $data[0]['declared_cost'];
        $this->paymentAmount = $data[0]['payment_amount'];
        $this->postCode = $data[0]['courier_delivery']['postcode'];
        $this->city = $data[0]['courier_delivery']['city'];
        $this->addressRecipient = $data[0]['courier_delivery']['address_recipient'];
        $this->issueType = $data[0]['issue_type'];

        if (!empty($data[0]['pickup_point'])) {
            $this->pointCode = $data[0]['pickup_point']['pickup_point_code'];
            $this->pointAddress = $data[0]['pickup_point']['pickup_point_address'];
        }


    }

    public function getItems(): array
    {
        return $this->items;
    }

    public function setItems(array $items): void
    {
        $this->items = $items;
    }

    public function getWeights(): array
    {
        return $this->weights;
    }

    public function setWeights(array $weights): void
    {
        $this->weights = $weights;
    }

    public function getFio(): string
    {
        return $this->fio;
    }

    public function setFio(string $fio): void
    {
        $this->fio = $fio;
    }

    public function getPhone(): string
    {
        return $this->phone;
    }

    public function setPhone(string $phone): void
    {
        $this->phone = $phone;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function setEmail(string $email): void
    {
        $this->email = $email;
    }

    public function getDeliveryType(): string
    {
        return $this->deliveryType;
    }

    public function setDeliveryType(string $deliveryType): void
    {
        $this->deliveryType = $deliveryType;
    }

    public function getDeliveryCost(): string
    {
        return $this->deliveryCost;
    }

    public function setDeliveryCost(string $deliveryCost): void
    {
        $this->deliveryCost = $deliveryCost;
    }

    public function getPaymentAmount(): string
    {
        return $this->paymentAmount;
    }

    public function setPaymentAmount(string $paymentAmount): void
    {
        $this->paymentAmount = $paymentAmount;
    }

    public function getPostCode(): ?string
    {
        return $this->postCode;
    }

    public function setPostCode(?string $postCode): void
    {
        $this->postCode = $postCode;
    }

    public function getCity(): string
    {
        return $this->city;
    }

    public function setCity(string $city): void
    {
        $this->city = $city;
    }

    public function getAddressRecipient(): ?string
    {
        return $this->addressRecipient;
    }

    public function setAddressRecipient(?string $addressRecipient): void
    {
        $this->addressRecipient = $addressRecipient;
    }

    public function getPointCode(): ?string
    {
        return $this->pointCode;
    }

    public function setPointCode(?string $pointCode): void
    {
        $this->pointCode = $pointCode;
    }

    public function getPointAddress(): ?string
    {
        return $this->pointAddress;
    }

    public function setPointAddress(?string $pointAddress): void
    {
        $this->pointAddress = $pointAddress;
    }

    public function getDeclaredCost(): string
    {
        return $this->declaredCost;
    }

    public function setDeclaredCost(string $declaredCost): void
    {
        $this->declaredCost = $declaredCost;
    }

    public function getIssueType(): int
    {
        return $this->issueType;
    }

    public function setIssueType(int $issueType): void
    {
        $this->issueType = $issueType;
    }

}