<?php

namespace Boxberry\Api\Entity;

class ParcelParams
{
    public const PICKUP_VID_ID = 1;
    public const COURIER_VID_ID = 2;
    public const ISSUE_TYPE_WITHOUT_OPENING_ID = '0';
    public const ISSUE_TYPE_WITH_OPENING_AND_VERIFICATION_ID = '1';
    public const ISSUE_TYPE_PARTIAL_ID = '2';
    public const EAEU_COURIER_DEFAULT_INDEX = '151';
    public const TRANSPORTER_GUID = 'fd85a8b6-4688-404f-9993-30b9e55d2950';
    public const ORDER_UPDATE_AVAILABLE_STATUSES_ID = [1];
    public const ORDER_DELETE_AVAILABLE_STATUSES_ID = [1, 10];
    public const ORDER_CANCEL_AVAILABLE_STATUSES_ID = [10, 76, 77, 80, 90, 100, 110, 120, 125, 140];
    public const ORDER_CHANGE_STORAGE_DATE_AVAILABLE_STATUSES_ID = [110, 140];
    private const PARCEL_CREATE_API_METHOD_NAME = 'ParselCreate';
    private const SOURCE_PLATFORM = 'bitrix_new';
    private string $updateByTrack = '';
    private string $orderId = '';
    private string $price = '';
    private string $paymentSum = '';
    private string $deliverySum = '';
    private string $vid = '';
    private string $transporterGuid = '';
    private string $pointName = '';
    private string $pointName1 = '';
    private string $fio = '';
    private string $phone = '';
    private string $email = '';
    private string $issue = '';
    private array $items = [];
    private array $weights = [];
    private ?Kurdost $kurdost = null;
    private ?Export $export = null;
    private ?Shop $shop = null;
    private ?Customer $customer = null;

    public function toArray(): array
    {
        $parcel = [
            'method' => self::PARCEL_CREATE_API_METHOD_NAME,
            'sdata' => [
                'source_platform' => self::SOURCE_PLATFORM,
                'updateByTrack' => $this->updateByTrack,
                'order_id' => $this->orderId,
                'price' => $this->price,
                'payment_sum' => $this->paymentSum,
                'delivery_sum' => $this->deliverySum,
                'vid' => $this->vid,
                'items' => $this->items,
                'weights' => $this->weights,
            ]
        ];

        if ($this->issue !== '') {
            $parcel['sdata']['issue'] = $this->issue;
        }

        if ($this->kurdost) {
            $parcel['sdata']['kurdost'] = [
                'index' => $this->kurdost->getIndex(),
                'citi' => $this->kurdost->getCiti(),
                'addressp' => $this->kurdost->getAddressp(),
            ];
        }

        if ($this->export) {
            $parcel['sdata']['export'] = [
                'index' => $this->export->getIndex(),
                'countryCode' => $this->export->getCountryCode(),
                'cityCode' => $this->export->getCityCode(),
                'area' => $this->export->getArea(),
                'street' => $this->export->getStreet(),
                'house' => $this->export->getHouse(),
                'flat' => $this->export->getFlat(),
                'transporterGuid' => $this->export->getTransporterGuid(),
            ];
        }

        if ($this->shop) {
            $parcel['sdata']['shop'] = [
                'name' => $this->shop->getName(),
                'name1' => $this->shop->getName1(),
            ];
        }

        if ($this->customer) {
            $parcel['sdata']['customer'] = [
                'fio' => $this->customer->getFio(),
                'phone' => $this->customer->getPhone(),
                'email' => $this->customer->getEmail(),
                'name' => $this->customer->getName(),
            ];
        }

        return $parcel;

    }

    public function getUpdateByTrack(): string
    {
        return $this->updateByTrack;
    }

    public function setUpdateByTrack(string $updateByTrack): void
    {
        $this->updateByTrack = $updateByTrack;
    }

    public function getOrderId(): string
    {
        return $this->orderId;
    }

    public function setOrderId(string $orderId): void
    {
        $this->orderId = $orderId;
    }

    public function getPrice(): string
    {
        return $this->price;
    }

    public function setPrice(string $price): void
    {
        $this->price = $price;
    }

    public function getPaymentSum(): string
    {
        return $this->paymentSum;
    }

    public function setPaymentSum(string $paymentSum): void
    {
        $this->paymentSum = $paymentSum;
    }

    public function getDeliverySum(): string
    {
        return $this->deliverySum;
    }

    public function setDeliverySum(string $deliverySum): void
    {
        $this->deliverySum = $deliverySum;
    }

    public function getVid(): string
    {
        return $this->vid;
    }

    public function setVid(string $vid): void
    {
        $this->vid = $vid;
    }

    public function getTransporterGuid(): string
    {
        return $this->transporterGuid;
    }

    public function setTransporterGuid(string $transporterGuid): void
    {
        $this->transporterGuid = $transporterGuid;
    }

    public function getPointName(): string
    {
        return $this->pointName;
    }

    public function setPointName(string $pointName): void
    {
        $this->pointName = $pointName;
    }

    public function getPointName1(): string
    {
        return $this->pointName1;
    }

    public function setPointName1(string $pointName1): void
    {
        $this->pointName1 = $pointName1;
    }

    public function getFio(): string
    {
        return $this->fio;
    }

    public function setFio(string $fio): void
    {
        $this->fio = $fio;
    }

    public function getPhone(): string
    {
        return $this->phone;
    }

    public function setPhone(string $phone): void
    {
        $this->phone = $phone;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function setEmail(string $email): void
    {
        $this->email = $email;
    }

    public function getIssue(): string
    {
        return $this->issue;
    }

    public function setIssue(string $issue): void
    {
        $this->issue = $issue;
    }

    public function getItems(): array
    {
        return $this->items;
    }

    public function setItems(array $items): void
    {
        $this->items = $items;
    }

    public function getWeights(): array
    {
        return $this->weights;
    }

    public function setWeights(array $weights): void
    {
        $this->weights = $weights;
    }

    public function getKurdost(): Kurdost
    {
        return $this->kurdost;
    }

    public function setKurdost(Kurdost $kurdost): void
    {
        $this->kurdost = $kurdost;
    }

    public function getShop(): Shop
    {
        return $this->shop;
    }

    public function setShop(Shop $shop): void
    {
        $this->shop = $shop;
    }

    public function getCustomer(): Customer
    {
        return $this->customer;
    }

    public function setCustomer(Customer $customer): void
    {
        $this->customer = $customer;
    }

    public function getExport(): ?Export
    {
        return $this->export;
    }

    public function setExport(?Export $export): void
    {
        $this->export = $export;
    }
}