<?php

namespace Boxberry\Api\Entity;

use Boxberry\Api\Exceptions\BoxberryApiException;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class WidgetSettings
{
    private array $hidePointsCityCodes;
    private array $hidePointsCodes;
    private int $isHideDeliveryDays;
    private int $addDeliveryDays;

    /**
     * @throws BoxberryApiException
     */
    public function __construct($data)
    {
        if (empty($data)) {
            throw new BoxberryApiException(Loc::getMessage('EMPTY_RESULT_FROM_WIDGET_SETTINGS'));
        }

        if (isset($data['err'])) {
            throw new BoxberryApiException($data['err']);
        }

        if (isset($data['error'])) {
            throw new BoxberryApiException($data['error']);
        }

        $this->hidePointsCityCodes = $data['result'][1]['CityCode'];
        $this->hidePointsCodes = $data['result'][1]['Code'];
        $this->isHideDeliveryDays = $data['result'][3]['hide_delivery_day'];
        $this->addDeliveryDays = $data['result'][3]['add_delivery_day'];
    }

    public function getHidePointsCityCodes(): array
    {
        return $this->hidePointsCityCodes;
    }

    public function setHidePointsCityCodes(array $hidePointsCityCodes): void
    {
        $this->hidePointsCityCodes = $hidePointsCityCodes;
    }

    public function getHidePointsCodes(): array
    {
        return $this->hidePointsCodes;
    }

    public function setHidePointsCodes(array $hidePointsCodes): void
    {
        $this->hidePointsCodes = $hidePointsCodes;
    }

    public function getIsHideDeliveryDays(): int
    {
        return $this->isHideDeliveryDays;
    }

    public function setIsHideDeliveryDays(int $isHideDeliveryDays): void
    {
        $this->isHideDeliveryDays = $isHideDeliveryDays;
    }

    public function getAddDeliveryDays(): int
    {
        return $this->addDeliveryDays;
    }

    public function setAddDeliveryDays(int $addDeliveryDays): void
    {
        $this->addDeliveryDays = $addDeliveryDays;
    }

    public function isCityDisabled($cityCode): bool
    {
        return in_array($cityCode, $this->hidePointsCityCodes);
    }

    public function isHideDeliveryDays(): bool
    {
        return $this->isHideDeliveryDays === 1;
    }

}