<?php
namespace Boxberry\Bitrix;

use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ORM\Data\DataManager;
use Bitrix\Main\ORM\Fields\DatetimeField;
use Bitrix\Main\ORM\Fields\IntegerField;
use Bitrix\Main\ORM\Fields\StringField;
use Bitrix\Main\ORM\Fields\TextField;
use Bitrix\Main\ORM\Fields\Validators\LengthValidator;
use Bitrix\Main\Result;
use Bitrix\Main\Error;
use Bitrix\Sale\Delivery\Services\Manager;
use Boxberry\Api\Client;
use Boxberry\Bitrix\Agents\Tracking;

Loc::loadMessages(__FILE__);

class OrdersTable extends DataManager
{
    /**
     * Returns DB table name for entity.
     *
     * @return string
     */
    public static function getTableName()
    {
        return 'b_boxberry_orders';
    }

    /**
     * Returns entity map definition.
     *
     * @return array
     */
    public static function getMap()
    {
        return [
            (new IntegerField('ID'))
                ->configurePrimary(true)
                ->configureAutocomplete(true)
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ID_FIELD')),

            (new StringField('SITE_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_SITE_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ORDER_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ORDER_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ACCOUNT_NUMBER'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ACCOUNT_NUMBER_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('DELIVERY_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_DELIVERY_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('PARENT_DELIVERY_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_PARENT_DELIVERY_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('BITRIX_STATUS_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_BITRIX_STATUS_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('BB_STATUS_ID'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_BB_STATUS_ID_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),
            (new StringField('BB_STATUS_DATE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_BB_STATUS_DATE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('BB_STATUS_NAME'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_BB_STATUS_NAME_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('TRACK'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_TRACK_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('STICKERS'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_STICKERS_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ACT'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ACT_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ACT_FILE_NAME'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ACT_FILE_NAME_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('DECLARED_COST'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_DECLARED_COST_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('PAYMENT_AMOUNT'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_PAYMENT_AMOUNT_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('DELIVERY_COST'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_DELIVERY_COST_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('DELIVERY_TYPE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_DELIVERY_TYPE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('POSTCODE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_POSTCODE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new StringField('CITY'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_CITY_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ADDRESS'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ADDRESS_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('PICKUP_POINT_CODE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_PICKUP_POINT_CODE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('POINT_FOR_PARCEL'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_POINT_FOR_PARCEL_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('FULL_NAME'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_FULL_NAME_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('EMAIL'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_EMAIL_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('PHONE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_PHONE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('CONTACT_PERSON'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_CONTACT_PERSON_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('COMPANY'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_COMPANY_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ERROR'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ERROR_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 255)),

            (new StringField('ISSUE_TYPE'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_ISSUE_TYPE_FIELD'))
                ->configureNullable(true)
                ->addValidator(new LengthValidator(null, 128)),

            (new TextField('PLACES'))
                ->configureTitle(Loc::getMessage('ORDERS_ENTITY_PLACES_FIELD'))
                ->configureNullable(true),
        ];
    }

    public static function getPaymentGroupNames(): array
    {
        return [
            'PAYMENT_AMOUNT' => Loc::getMessage('ORDERS_ENTITY_PAYMENT_AMOUNT_FIELD'),
            'DELIVERY_COST' => Loc::getMessage('ORDERS_ENTITY_DELIVERY_COST_FIELD'),
            'DECLARED_COST' => Loc::getMessage('ORDERS_ENTITY_DECLARED_COST_FIELD')
        ];
    }

    public static function getClientGroupNames(): array
    {
        return [
            'PHONE' => Loc::getMessage('ORDERS_ENTITY_PHONE_FIELD'),
            'EMAIL' => Loc::getMessage('ORDERS_ENTITY_EMAIL_FIELD')
        ];
    }

    public static function getPickupGroupNames(): array
    {
        return [
            'PICKUP_POINT_CODE' => Loc::getMessage('ORDERS_ENTITY_PICKUP_POINT_CODE_FIELD'),
        ];
    }

    public static function getCourierGroupNames(): array
    {
        return [
            'POSTCODE' => Loc::getMessage('ORDERS_ENTITY_POSTCODE_FIELD'),
        ];
    }

    public static function getAddressGroupNames(): array
    {
        return [
            'ADDRESS' => Loc::getMessage('ORDERS_ENTITY_ADDRESS_FIELD'),
        ];
    }

    public static function getOrderIdsGroupNames(): array
    {
        return [
            'ORDER_ID' => Loc::getMessage('ORDERS_ENTITY_ORDER_ID_FIELD'),
           // 'ACCOUNT_NUMBER' => Loc::getMessage('ORDERS_ENTITY_ACCOUNT_NUMBER_FIELD')
        ];
    }

    public static function getDeliveryIdsGroupNames(): array
    {
        return [
            'DELIVERY_ID' => Loc::getMessage('ORDERS_ENTITY_DELIVERY_ID_FIELD'),
            'PARENT_DELIVERY_ID' => Loc::getMessage('ORDERS_ENTITY_PARENT_DELIVERY_ID_FIELD'),
        ];
    }

    public static function getPointForParcelsGroupNames(): array
    {
        return [
            'POINT_FOR_PARCEL' => Loc::getMessage('ORDERS_ENTITY_POINT_FOR_PARCEL_FIELD')
        ];
    }

    public static function getOrderIdName(): string
    {
        return 'ORDER_ID';
    }

    public static function getApTokensAndTracksByOrderIds(array $orderIds): Result
    {
        $result = new Result();

        try {
            $ordersData = self::getTracksAndParentDeliveryIdsByOrderIds($orderIds);

            if (!$ordersData->isSuccess()) {
                return $ordersData;
            } else {
                $ordersData = $ordersData->getData();
            }

            $deliveryIds = array_column($ordersData, 'PARENT_DELIVERY_ID');
            $apiTokensData = Manager::getList([
                'filter' => [
                    'ID' => $deliveryIds
                ],
                'select' => ['ID', 'CONFIG']
            ])->fetchAll();

            if (!$apiTokensData) {
                $result->addError(new Error(Loc::getMessage('API_TOKENS_FOR_SELECTED_ORDERS_NOT_FOUND'), 0, ['orderIds' => $orderIds]));
                return $result;
            }

            $apiTokensAndTracksArray = [];
            foreach ($ordersData as $item) {
                foreach ($apiTokensData as $apiToken) {
                    if ($item['PARENT_DELIVERY_ID'] == $apiToken['ID']) {
                        $apiTokenKey = $apiToken['CONFIG']['MAIN']['API_KEY'];
                        $found = false;
                        foreach ($apiTokensAndTracksArray as &$existingArray) {
                            if ($existingArray['apiToken'] === $apiTokenKey) {
                                $existingArray['tracks'][] = $item['TRACK'];
                                $existingArray['orderIds'][] = $item['ORDER_ID'];
                                $found = true;
                                break;
                            }
                        }
                        if (!$found) {
                            $apiTokensAndTracksArray[] = [
                                'apiToken' => $apiTokenKey,
                                'tracks' => [$item['TRACK']],
                                'orderIds' => [$item['ORDER_ID']],
                            ];
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        $result->setData($apiTokensAndTracksArray);
        return $result;
    }

    public static function saveActToOrders(array $actLinksAndOrderIds): Result
    {
        $result = new Result();

        if (empty($actLinksAndOrderIds)) {
            $result->addError(new Error(Loc::getMessage('EMPTY_ARRAY_WITH_ACTS')));
            return $result;
        }

        foreach ($actLinksAndOrderIds as $actLinksAndOrderId) {
            try {
                $ordersData = self::query()
                    ->setSelect(['ID'])
                    ->whereIn('ORDER_ID', $actLinksAndOrderId['orderIds'])
                    ->exec()
                    ->fetchAll();

                if (!$ordersData) {
                    $result->addError(new Error(Loc::getMessage('ORDERS_NOT_FOUND_WITH_COLON') . implode(', ', $actLinksAndOrderId['orderIds']), 0, ['orderIds' => $actLinksAndOrderId['orderIds']]));
                    return $result;
                }

                $primaries = [];
                foreach ($ordersData as $orderData) {
                    $primaries[] = ['ID' => $orderData['ID']];
                }

                if (empty($primaries)) {
                    $result->addError(new Error(Loc::getMessage('ORDER_IDS_FOR_UPDATE_NOT_FOUND')));
                    return $result;
                }

                $updateResult = self::updateMulti(
                    $primaries,
                    ['ACT' => $actLinksAndOrderId['actUrl'], 'ACT_FILE_NAME' => Client::makeActFileNameByOrderIds($actLinksAndOrderId['orderIds'])],
                    true
                );

                if (!$updateResult->isSuccess()) {
                    $result->addErrors($updateResult->getErrors());
                    return $result;
                }
            } catch (\Exception $e) {
                $result->addError(new Error($e->getMessage()));
                return $result;
            }
        }

        return $result;
    }

    public static function isOrderWithBbDelivery($orderId): bool
    {
        try {
            $data = self::query()
                ->setSelect(['ORDER_ID'])
                ->where('ORDER_ID', $orderId)
                ->exec()
                ->fetch();

            return (bool)$data;
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getStickersUrlByOrderId($orderId): Result
    {
        $result = new Result();

        try {
            $data = self::query()
                ->setSelect(['STICKERS'])
                ->where('ORDER_ID', $orderId)
                ->exec()
                ->fetch();

            if ($data) {
                $result->setData($data);
            } else {
                $result->addError(new Error(Loc::getMessage('STICKERS_NOT_FOUND')));
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        return $result;
    }

    public static function getActUrlByOrderId($orderId): Result
    {
        $result = new Result();

        try {
            $data = self::query()
                ->setSelect(['ACT'])
                ->where('ORDER_ID', $orderId)
                ->exec()
                ->fetch();

            if ($data) {
                $result->setData($data);
            } else {
                $result->addError(new Error(Loc::getMessage('ACT_NOT_FOUND')));
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        return $result;
    }

    public static function getActFileNamesByOrderIds($orderIds): Result
    {
        $result = new Result();

        try {
            $data = self::query()
                ->setSelect(['ORDER_ID', 'ACT_FILE_NAME'])
                ->whereIn('ORDER_ID', $orderIds)
                ->exec()
                ->fetchAll();

            if ($data) {
                $orderIds = array_column($data, 'ORDER_ID');
                $actFileNames = array_column($data, 'ACT_FILE_NAME');
                $result->setData(array_combine($orderIds, $actFileNames));
            } else {
                $result->addError(new Error(Loc::getMessage('ACTS_NOT_FOUND')));
            }
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        return $result;
    }

    public static function saveActFileName($orderId, $actFileName): bool
    {
        try {
            $data = self::query()
                ->setSelect(['ID'])
                ->where('ORDER_ID', $orderId)
                ->exec()
                ->fetch();

            if ($data) {
                $updateResult = self::update($data['ID'], ['ACT_FILE_NAME' => $actFileName]);
                return $updateResult->isSuccess();
            } else {
                return false;
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getOrdersIdsByTracks($tracks): Result
    {
        $result = new Result();

        try {
            $data = self::query()
                ->setSelect(['ORDER_ID'])
                ->whereIn('TRACK', $tracks)
                ->exec()
                ->fetchAll();

            if (empty($data)) {
                $result->addError(new Error(Loc::getMessage('ORDERS_WITH_SELECTED_TRACKS_NOT_FOUND'), 0, ['tracks' => $tracks]));
                return $result;
            }

            $orderIds = array_map(function($item) {
                return $item['ORDER_ID'];
            }, $data);

            $result->setData($orderIds);
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        return $result;
    }

    public static function getTracksAndParentDeliveryIdsByOrderIds($orderIds): Result
    {
        $result = new Result();

        try {
            $ordersData = self::query()
                ->setSelect(['ORDER_ID', 'TRACK', 'PARENT_DELIVERY_ID'])
                ->whereIn('ORDER_ID', $orderIds)
                ->whereNotNull('TRACK')
                ->exec()
                ->fetchAll();
        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
            return $result;
        }

        if (!$ordersData) {
            $result->addError(new Error(Loc::getMessage('ALL_SELECTED_ORDERS_WITHOUT_TRACKS'), 0, ['orderIds' => $orderIds]));
            return $result;
        }

        $presentOrderIds = array_column($ordersData, 'ORDER_ID');
        $missingOrderIds = array_diff($orderIds, $presentOrderIds);

        if ($missingOrderIds) {
            $result->addError(new Error(Loc::getMessage('ORDERS_WITH_IDS') . implode(', ', $missingOrderIds) . Loc::getMessage('MISSING_TRACKS'), 0, ['orderIds' => $missingOrderIds]));
            return $result;
        }

        $result->setData($ordersData);

        return $result;

    }

    public static function getOrdersForStatusSync(int $ordersLimit, int $excludeDays): array
    {
        $date = new \DateTime();
        $date->modify("-{$excludeDays} days");
        $date = $date->format('Y-m-d H:i:s');

        return self::query()
            ->setSelect(['ID', 'ORDER_ID', 'DELIVERY_ID', 'PARENT_DELIVERY_ID', 'BITRIX_STATUS_ID', 'BB_STATUS_ID', 'TRACK', 'BB_STATUS_DATE'])
            ->whereNotNull('TRACK')
            ->whereNotNull('DELIVERY_ID')
            ->whereNotNull('PARENT_DELIVERY_ID')
            ->whereNotNull('BB_STATUS_ID')
            ->whereNotNull('BITRIX_STATUS_ID')
            ->whereNotNull('BB_STATUS_DATE')
            ->where('TRACK', '!=', '')
            ->where('DELIVERY_ID', '!=', '')
            ->where('PARENT_DELIVERY_ID', '!=', '')
            ->where('BB_STATUS_ID', '!=', '')
            ->where('BB_STATUS_ID', '!=', 0)
            ->where('BITRIX_STATUS_ID', '!=', '')
            ->where('BB_STATUS_DATE', '!=', '')
            ->where('BB_STATUS_DATE', '>', $date)
            ->where('BB_STATUS_ID', '!=', Tracking::BB_SUCCESS_FINAL_STATUS_ID)
            ->where('BB_STATUS_ID', '!=', Tracking::BB_RETURN_FINAL_STATUS_ID)
            ->setLimit($ordersLimit)
            ->exec()
            ->fetchAll();
    }


}