<?php

namespace Boxberry\Bitrix\Agents;

use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;
use Boxberry\Api\Client;

/**
 * Class BaseAgent
 *
 * Базовый класс для агентов выполняющих обновление данных из API Boxberry
 *
 */
abstract class BaseAgent
{
    protected string $agentName;

    public function update(): string
    {
        $bbDeliveryIds = Delivery::getAllProfilesIds();
        $isEventLogEnabled = Options::isEventLogEnabled();
        $updated = false;

        if (empty($bbDeliveryIds)) {
            return $this->agentName;
        }

        foreach ($bbDeliveryIds as $bbDeliveryId) {

            if ($updated) {
                break;
            }

            try {
                $options = new Options($bbDeliveryId);
            } catch (\Exception $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), $this->agentName);
                }

                continue;
            }

            $apiToken = !$options->isTestMode() ? $options->getApiKey() : Client::TEST_TOKEN;

            try {
                $data = $this->getData($apiToken);
            } catch (\Exception $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), $this->agentName);
                }

                continue;
            }

            try {
                $this->processData($data);
                $updated = true;
            } catch (\Exception $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), $this->agentName);
                }

                continue;
            }

        }

        return $this->agentName;
    }

    public static function run(): string
    {
        $instance = new static();
        return $instance->update();
    }

    abstract protected function getData($apiToken);

    abstract protected function processData($data);
}