<?php

namespace Boxberry\Bitrix\Agents;

use Bitrix\Main\Config\Option;
use Bitrix\Sale\Delivery\Services\Manager;
use Boxberry\Api\Client;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Logger;
use Boxberry\Bitrix\Helpers\Options;

class KeysChecker
{
    /**
     * Проверяет активность токена для каждого активного способа доставки, и в случае ошибки записывает ее в Журнал событий
     *
     * @return string
     */
    public static function checkToken(): string
    {
        $agentName = __METHOD__ . '();';
        $isEventLogEnabled = Options::isEventLogEnabled();
        $bbDeliveryIds = Delivery::getAllProfilesIds();

        if (empty($bbDeliveryIds)) {
            return $agentName;
        }

        $checkedDeliveries = [];

        foreach ($bbDeliveryIds as $bbDeliveryId) {

            try {
                $options = new Options($bbDeliveryId);
                if ($options->isTestMode()) {
                    continue;
                }

                if (in_array($options->getParentId(), $checkedDeliveries)) {
                    unset($options);
                    continue;
                }
            } catch (\Exception $e) {
                if ($isEventLogEnabled) {
                    Logger::Add($e->getMessage(), $agentName);
                }

                continue;
            }

            $apiToken = $options->getApiKey();

            try {
                $client = new Client($apiToken, 30);
                $client->getKeyIntegration();
                $checkedDeliveries[] = $options->getParentId();
            } catch (\Exception $e) {
                Logger::Add($e->getMessage(), $agentName);
                $checkedDeliveries[] = $options->getParentId();

                continue;
            }

        }

        return $agentName;
    }

}