<?php

namespace Boxberry\Bitrix\Entity;

use Bitrix\Main\ArgumentException;
use Bitrix\Main\ObjectPropertyException;
use Bitrix\Main\Result;
use Bitrix\Main\SystemException;
use Bitrix\Main\Text\StringHelper;
use Bitrix\Sale\ResultError;
use Boxberry\Bitrix\OrdersTable;
use Boxberry\Bitrix\PointsForParcelsTable;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class OrderData
{
    private ?string $siteId = null;
    private ?int $orderId = null;
    private ?string $accountNumber = null;
    private ?int $deliveryId = null;
    private ?int $parentDeliveryId = null;
    private ?string $bitrixStatusId = null;
    private ?int $bbStatusId = null;
    private ?string $bbStatusDate = null;
    private ?string $bbStatusName = null;
    private ?string $track = null;
    private ?string $stickers = null;
    private ?string $act = null;
    private ?string $declaredCost = null;
    private ?string $paymentAmount = null;
    private ?string $deliveryCost = null;
    private ?int $deliveryType = null;
    private ?string $postcode = null;
    private ?string $city = null;
    private ?string $address = null;
    private ?string $addressRecipient = null;
    private ?string $pickupPointCode = null;
    private ?string $pickupPointAddress = null;
    private ?string $pointForParcel = null;
    private ?string $fullName = null;
    private ?string $phone = null;
    private ?string $email = null;
    private ?string $contactPerson = null;
    private ?string $company = null;
    private ?string $error = null;
    private ?string $issueType = null;
    private ?string $places = null;

    public function getSiteId(): ?string
    {
        return $this->siteId;
    }

    public function getOrderId(): ?int
    {
        return $this->orderId;
    }

    public function getAccountNumber(): ?string
    {
        return $this->accountNumber;
    }

    public function getDeliveryId(): ?int
    {
        return $this->deliveryId;
    }

    public function getParentDeliveryId(): ?int
    {
        return $this->parentDeliveryId;
    }

    public function getBitrixStatusId(): ?string
    {
        return $this->bitrixStatusId;
    }

    public function getBbStatusId(): ?int
    {
        return $this->bbStatusId;
    }

    public function getBbStatusDate(): ?string
    {
        return $this->bbStatusDate;
    }

    public function getBbStatusName(): ?string
    {
        return $this->bbStatusName;
    }

    public function getTrack(): ?string
    {
        return $this->track;
    }

    public function getStickers(): ?string
    {
        return $this->stickers;
    }

    public function getAct(): ?string
    {
        return $this->act;
    }

    public function getDeclaredCost(): ?string
    {
        return $this->declaredCost;
    }

    public function getPaymentAmount(): ?string
    {
        return $this->paymentAmount;
    }

    public function getDeliveryCost(): ?string
    {
        return $this->deliveryCost;
    }

    public function getDeliveryType(): ?int
    {
        return $this->deliveryType;
    }

    public function getPostcode(): ?string
    {
        return $this->postcode;
    }

    public function getCity(): ?string
    {
        return $this->city;
    }

    public function getAddressRecipient(): ?string
    {
        return $this->addressRecipient;
    }

    public function getPickupPointCode(): ?string
    {
        return $this->pickupPointCode;
    }

    public function getPickupPointAddress(): ?string
    {
        return $this->pickupPointAddress;
    }

    public function getPointForParcel(): ?string
    {
        return $this->pointForParcel;
    }

    public function getFullName(): ?string
    {
        return $this->fullName;
    }

    public function getPhone(): ?string
    {
        return $this->phone;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function getError(): ?string
    {
        return $this->error;
    }

    public function getIssueType(): ?string
    {
        return $this->issueType;
    }

    public function setSiteId($siteId): void
    {
        $this->siteId = $siteId;
    }

    public function setOrderId($orderId): void
    {
        $this->orderId = $orderId;
    }

    public function setAccountNumber($accountNumber): void
    {
        $this->accountNumber = $accountNumber;
    }

    public function setDeliveryId($deliveryId): void
    {
        $this->deliveryId = $deliveryId;
    }

    public function setParentDeliveryId($parentDeliveryId): void
    {
        $this->parentDeliveryId = $parentDeliveryId;
    }

    public function setBitrixStatusId($bitrixStatusId): void
    {
        $this->bitrixStatusId = $bitrixStatusId;
    }

    public function setBbStatusId($bbStatusId): void
    {
        $this->bbStatusId = $bbStatusId;
    }

    public function setBbStatusDate($bbStatusDate): void
    {
        $this->bbStatusDate = $bbStatusDate;
    }

    public function setBbStatusName($bbStatusName): void
    {
        $this->bbStatusName = $bbStatusName;
    }

    public function setTrack($track): void
    {
        $this->track = $track;
    }

    public function setStickers($stickers): void
    {
        $this->stickers = $stickers;
    }

    public function setAct($act): void
    {
        $this->act = $act;
    }

    public function setDeclaredCost($declaredCost): void
    {
        $this->declaredCost = $declaredCost;
    }

    public function setPaymentAmount($paymentAmount): void
    {
        $this->paymentAmount = $paymentAmount;
    }

    public function setDeliveryCost($deliveryCost): void
    {
        $this->deliveryCost = $deliveryCost;
    }

    public function setDeliveryType($deliveryType): void
    {
        $this->deliveryType = $deliveryType;
    }

    public function setPostcode($postcode): void
    {
        $this->postcode = $postcode;
    }

    public function setCity($city): void
    {
        $this->city = $city;
    }

    public function setAddressRecipient($addressRecipient): void
    {
        $this->addressRecipient = $addressRecipient;
    }

    public function setPickupPointCode($pickupPointCode): void
    {
        $this->pickupPointCode = $pickupPointCode;
    }

    public function setPickupPointAddress($pickupPointAddress): void
    {
        $this->pickupPointAddress = $pickupPointAddress;
    }

    /**
     * @throws \Exception
     */
    public function setPointForParcel(?string $pointForParcel): void
    {
        if (!is_numeric($pointForParcel) && (!$pointForParcel = PointsForParcelsTable::getPointCodeByName($pointForParcel))) {
            throw new \Exception(Loc::getMessage('ERROR_CAN_NOT_GET_POINT_FOR_PARCEL_CODE_CHECK_NAME'));
        }

        $this->pointForParcel = $pointForParcel;
    }

    public function setFullName($fullName): void
    {
        $this->fullName = $fullName;
    }

    public function setPhone($phone): void
    {
        $this->phone = $phone;
    }

    public function setEmail($email): void
    {
        $this->email = $email;
    }

    public function setError($error): void
    {
        $this->error = $error;
    }

    public function setIssueType($issueType): void
    {
        $this->issueType = $issueType;
    }

    public function toArray(): array
    {
        $fieldObjects = OrdersTable::getMap();
        $properties = get_object_vars($this);
        $data = [];

        foreach ($fieldObjects as $fieldObject) {

            if ($fieldObject->isPrimary()) {
                continue;
            }

            $fieldName = $fieldObject->getName();
            $propName = lcfirst(StringHelper::snake2camel($fieldName));

            $data[$fieldName] = $properties[$propName];
        }

        return $data;

    }

    /**
     * @throws \Exception
     */
    public function setValues($orderId = 0, $replacePreviousValuesWithNull = false): bool
    {
        $data = $this->toArray();

        if (empty($data)) {
            throw new \Exception(Loc::getMessage('ERROR_EMPTY_ORDER_DATA_FOR_SET_VALUES'));
        }

        if (!$orderId) {
            if ($this->orderId) {
                $orderId = $this->orderId;
            } else {
                throw new \Exception(Loc::getMessage('ERROR_ORDER_ID_IS_NOT_SET'));
            }
        }

        if ($order = $this->getBbOrderByOrderId($orderId)) {

            if (!$replacePreviousValuesWithNull) {
                foreach ($data as $key => $value) {
                    if (is_null($value)) {
                        $data[$key] = $order[$key];
                    }
                }
            }

             return OrdersTable::update($order['ID'], $data)->isSuccess();
        } else {
           return OrdersTable::add($data)->isSuccess();
        }
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    public function getValues($orderId): bool
    {
        $result = new Result();
        $order = $this->getBbOrderByOrderId($orderId);

        if (!$order) {
            $result->addError(new ResultError(Loc::getMessage('ERROR_ORDER_NOT_FOUND_IN_ORDERS_TABLE')));
            return $result->isSuccess();
        }

        foreach ($order as $key => $value) {
            $propName = lcfirst(StringHelper::snake2camel($key));
            $this->$propName = $value;
        }

        return $result->isSuccess();
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    public function getBbOrderByOrderId($orderId): bool|array
    {
        return OrdersTable::query()
            ->setFilter(['ORDER_ID' => $orderId])
            ->addSelect('*')
            ->exec()
            ->fetch();
    }

    public function getPlaces(): ?string
    {
        return $this->places;
    }

    public function setPlaces(?string $places): void
    {
        $this->places = $places;
    }

    public function getAddress(): ?string
    {
        return $this->address;
    }

    public function setAddress(?string $address): void
    {
        $this->address = $address;
    }

    public function getContactPerson(): ?string
    {
        return $this->contactPerson;
    }

    public function setContactPerson(?string $contactPerson): void
    {
        $this->contactPerson = $contactPerson;
    }

    public function getCompany(): ?string
    {
        return $this->company;
    }

    public function setCompany(?string $company): void
    {
        $this->company = $company;
    }

}