<?php

namespace Boxberry\Bitrix\Forms;

use Bitrix\Main\Text\StringHelper;
use Bitrix\Sale\Order;
use Boxberry\Bitrix\Entity\OrderData;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\Helpers\Options;
use Boxberry\Bitrix\OrdersTable;
use CAdminForm;
use CFile;
use CIBlockElement;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class OrderForm
{
    private array $size;
    private CAdminForm $form;
    private OrderData $bbOrder;
    private Order $order;
    private array $bbProfiles;

    public function __construct(Order $order)
    {
        $this->size = ['size' => 20];
        $this->bbOrder = new OrderData();

        try {
            $this->bbOrder->getValues($order->getId());
        } catch (\Exception $e) {
        }

        $this->bbProfiles = Delivery::getProfilesIdsWithNames();
        $this->form = new \CAdminForm('order_form', [
            ['DIV' => 'order_form', 'TAB' => Loc::getMessage('DELIVERY_TAB_NAME'), 'TITLE' => Loc::getMessage('DELIVERY_TAB_DESCRIPTION')],
            ['DIV' => 'order_form2', 'TAB' => Loc::getMessage('RECIPIENT_TAB_NAME'), 'TITLE' => Loc::getMessage('RECIPIENT_TAB_DESCRIPTION')],
            ['DIV' => 'order_form3', 'TAB' => Loc::getMessage('PLACES_TAB_NAME'), 'TITLE' => Loc::getMessage('PLACES_TAB_DESCRIPTION')],
            ['DIV' => 'order_form4', 'TAB' => Loc::getMessage('STORAGE_PERIOD_TAB_NAME'), 'TITLE' => Loc::getMessage('STORAGE_PERIOD_TAB_DESCRIPTION')]
        ], false);
        $this->order = $order;
    }

    public function show(): void
    {
        $this->form->BeginPrologContent();
        echo '<input type="hidden" name="lang" value="' . LANGUAGE_ID . '">';
        echo bitrix_sessid_post();
        $this->form->EndPrologContent();
        $this->form->BeginEpilogContent();
        echo bitrix_sessid_post();
        echo '<input type="hidden" name="lang" value="' . LANGUAGE_ID . '">';
        $this->form->EndEpilogContent();
        echo '<div id="ajax_status_message" style="display: none"></div>';

        $this->form->Begin(['FORM_ACTION' => $GLOBALS['APPLICATION']->GetCurPage() . '?lang=' . LANGUAGE_ID]);
        $this->form->BeginNextFormTab();
        $this->makeHiddenFields();
        $this->form->AddViewField('ACCOUNT_NUMBER_VIEW', OrdersTable::getEntity()->getField('ACCOUNT_NUMBER')->getTitle(), $this->order->getField('ACCOUNT_NUMBER'));
        $this->form->AddDropDownField('DELIVERY_ID_LIST', Loc::getMessage('DELIVERY_ID_LIST'), true, $this->bbProfiles, $this->bbOrder->getDeliveryId() ?: '');

        $this->addEditFields(OrdersTable::getOrderIdsGroupNames());

        $this->form->AddDropDownField('ISSUE_TYPE', Loc::getMessage('ISSUE_TYPE'), false, Options::getIssueTypesArray(), $this->bbOrder->getIssueType() ?: '');

        $this->addEditFields(OrdersTable::getPointForParcelsGroupNames());
        $this->addEditFields(OrdersTable::getPaymentGroupNames());
        $this->addEditFields(OrdersTable::getPickupGroupNames());
        $this->addEditFields(OrdersTable::getAddressGroupNames());

        $this->form->sButtonsContent = '<input id="bbOpenWidget" type="submit" name="save" value="' . Loc::getMessage('SELECT_PVZ') . '" title="' . Loc::getMessage('SELECT_PVZ') . '" class="adm-btn">';

        $this->addEditFields(OrdersTable::getCourierGroupNames());

        $this->form->BeginNextFormTab();

        $payerType = Options::getPayerTypeId($this->order->getPersonTypeId());

        if ($payerType === Options::FIZ_PAYER_TYPE_ID) {
            $this->form->AddEditField('FULL_NAME', Loc::getMessage('FULL_NAME'), false, $this->size, $this->bbOrder->getFullName() ?: '');
        }

        if ($payerType === Options::JUR_PAYER_TYPE_ID) {
            $this->form->AddEditField('COMPANY', Loc::getMessage('COMPANY'), false, $this->size, $this->bbOrder->getCompany() ?: '');
            $this->form->AddEditField('CONTACT_PERSON', Loc::getMessage('CONTACT_PERSON'), false, $this->size, $this->bbOrder->getContactPerson() ?: '');

        }

        $this->addEditFields(OrdersTable::getClientGroupNames());

        $this->form->BeginNextFormTab();

        $this->form->BeginCustomField('PRODUCTS', '');
        echo '<tr><td colspan="2" id="tableContainer"></td></tr>';
        $this->form->EndCustomField('PRODUCTS');
        $this->form->BeginNextFormTab();
        $this->form->AddViewField('STORAGE_DATE', Loc::getMessage('CURRENT_STORAGE_DATE'), Loc::getMessage('NOT_SET'));
        $value = date('d.m.Y', strtotime('+1 day'));
        $this->form->AddCalendarField('NEW_STORAGE_DATE', Loc::getMessage('CHOOSE_DATE'), $value);
        $this->form->BeginCustomField('bbChangeOrderStorageDateButton', '');
        ?>
        <tr>
            <td ></td>
            <td >
                <input id="bbChangeOrderStorageDate" type="submit" name="bbChangeOrderStorageDate" value="<?php echo Loc::getMessage('CHANGE_ORDER_STORAGE_DATE_BUTTON_TEXT')?>" title="<?php echo Loc::getMessage('CHANGE_ORDER_STORAGE_DATE_BUTTON_TEXT')?>">
            </td>
        </tr>
        <?php
        $this->form->EndCustomField('bbChangeOrderStorageDateButton');

        if (!$this->bbOrder->getAct() && $this->bbOrder->getTrack()) {
            $saveButtonTitle = Loc::getMessage('UPDATE_ORDER_IN_LK');
        } elseif (!$this->bbOrder->getAct() && !$this->bbOrder->getTrack()) {
            $saveButtonTitle = Loc::getMessage('CREATE_ORDER_IN_LK');
        } else {
            $saveButtonTitle = Loc::getMessage('UPDATE_ORDER_IN_LK');
        }

        $this->form->sButtonsContent .= '<input id="bbSaveButton" type="submit" value="' . $saveButtonTitle . '" title="' . $saveButtonTitle . '"class="adm-btn-save" >';
        $this->form->sButtonsContent .= '<input id="bbUpdateStatusButton" type="submit" style="float: right;" name="updateStatus" value="' . Loc::getMessage('UPDATE_STATUS_BUTTON') . '" title="' . Loc::getMessage('UPDATE_STATUS_BUTTON') . '" class="adm-btn-delete">';
        $this->form->sButtonsContent .= '<input id="bbRecalculateButton" type="submit" style="float: right;" name="recalculate" value="' . Loc::getMessage('RECALCULATE_BUTTON') . '" title="' . Loc::getMessage('RECALCULATE_BUTTON') . '" class="adm-btn-delete">';
        $this->form->sButtonsContent .= '<input id="bbDeleteButton" type="submit" style="float: right;" name="delete"  value="' . Loc::getMessage('DELETE_BUTTON') . '" title="' . Loc::getMessage('DELETE_BUTTON') . '" class="adm-btn-delete">';
        $this->form->sButtonsContent .= '<input id="bbCancelButton" type="submit" style="float: right;" name="cancel" value="' . Loc::getMessage('CANCEL_BUTTON') . '" title="' . Loc::getMessage('CANCEL_BUTTON') . '" class="adm-btn-delete">';
        $this->form->ShowTabButtons();
        $this->form->Show();
    }

    private function makeHiddenField($name, $value = ''): void
    {
        $this->form->BeginCustomField($name, '');
        echo '<tr><td colspan="2">';
        echo '<input id="' . $name . '" name="' . $name . '" type="hidden" value="' . $value . '" size="50">';
        echo '</td></tr>';
        $this->form->EndCustomField($name);
    }

    private function makeHiddenFields(): void
    {
        $hiddenFields = [
            'WIDGET_KEY' => '',
            'CITY_NAME' => '',
            'CITY' => '',
            'DELIVERY_TYPE' => '',
            'ORDER_ID' => $this->order->getId(),
            'DELIVERY_ID' => $this->bbOrder->getDeliveryId() ?: '',
            'PARENT_DELIVERY_ID' => $this->bbOrder->getParentDeliveryId() ?: '',
            'SITE_ID' => $this->order->getSiteId() ?: '',
            'BITRIX_STATUS_ID' => $this->order->getField('STATUS_ID') ?: '',
            'CONSOLE_LOG_ENABLED' => Options::isConsoleLogEnabled() ? 'Y' : 'N',
            'ACCOUNT_NUMBER' => $this->order->getField('ACCOUNT_NUMBER') ?: '',
            'STORE_DATE' => '',
        ];

        foreach ($hiddenFields as $name => $value) {
            $this->makeHiddenField($name, $value);
        }
    }

    private function addEditFields(array $groupNames): void
    {
        foreach ($groupNames as $groupName => $groupValue) {
            if ($groupName === OrdersTable::getOrderIdName()) {
                continue;

            }
            $this->form->AddEditField($groupName, $groupValue, false, $this->size, $this->bbOrder->{'get' . StringHelper::snake2camel($groupName)}() ?: '');
        }
    }

    private static function getProductResizeImage($productId): string
    {
        $arProduct = CIBlockElement::GetByID($productId)->GetNext();

        if (isset($arProduct['DETAIL_PICTURE'])) {
            $imagePath = CFile::ResizeImageGet($arProduct['DETAIL_PICTURE'], ['width' => 100, 'height' => 100])['src'];

            if ($imagePath) {
                echo '<td><img src="' . $imagePath . '" alt=""></td>';
            }
        }

        return '';
    }


}