<?php

namespace Boxberry\Bitrix\Helpers;

use Boxberry\Bitrix\Entity\ProductItem;
use Boxberry\Bitrix\Helpers\BoxSizes;
use Boxberry\Bitrix\Helpers\Options;
use Boxberry\Api\CountriesSettings;

class BoxSizesCalculator
{
    public static function getBoxSizesArrayFromOrderItems($orderItems, $deliveryId, $calculate = true): array
    {
        $box = new BoxSizes();
        $options = new Options($deliveryId);
        $defaultWeight = empty($options->getDefaultWeight()) ? Delivery::BB_DEFAULT_WEIGHT : intval(ceil($options->getDefaultWeight()));
        $defaultWidth = empty($options->getDefaultWidth()) ? 0 : intval(ceil($options->getDefaultWidth()));
        $defaultHeight = empty($options->getDefaultHeight()) ? 0 : intval(ceil($options->getDefaultHeight()));
        $defaultDepth = empty($options->getDefaultDepth()) ? 0 : intval(ceil($options->getDefaultDepth()));

        if ($options->isSetDefaultsForOrder()) {
            $box->addBox($defaultWidth, $defaultHeight, $defaultDepth, $defaultWeight);
        } else {
            $lastItem = end($orderItems);

            foreach ($orderItems as $item) {

                $dimensions = unserialize($item['DIMENSIONS']);

                $weight = empty($item['WEIGHT']) ? $defaultWeight : intval(ceil($item['WEIGHT']));
                $width = empty($dimensions['WIDTH']) ? $defaultWidth : intval(ceil($dimensions['WIDTH'])) / 10;
                $height = empty($dimensions['HEIGHT']) ? $defaultHeight : intval(ceil($dimensions['HEIGHT'])) / 10;
                $depth = empty($dimensions['LENGTH']) ? $defaultDepth : intval(ceil($dimensions['LENGTH'])) / 10;
                $id = $item['PRODUCT_ID'];
                $name = $item['NAME'];
                $price = $item['PRICE'];
                $quantity = $item['QUANTITY'];

                $item = new ProductItem($weight, $width, $height, $depth, $id, $name, $price, $quantity);

                if ($item->weight <= CountriesSettings::LOAD_LIMIT &&
                    $item->x + $item->y + $item->z <= CountriesSettings::MAX_SUM_ALL_SIDES &&
                    $item->x <= CountriesSettings::MAX_LENGTH_SIDE &&
                    $item->y <= CountriesSettings::MAX_LENGTH_SIDE &&
                    $item->z <= CountriesSettings::MAX_LENGTH_SIDE) {
                    for ($i = 0; $i < $item->quantity; $i++) {
                        if ($box->getWeight() + ceil($item->weight) > CountriesSettings::LOAD_LIMIT ||
                            $box->getLengthSide(ceil($item->x), ceil($item->y), ceil($item->z)) * 3 > CountriesSettings::MAX_SUM_ALL_SIDES) {
                            $box->addBox($box->getWidth(), $box->getHeight(), $box->getDepth(), $box->getWeight());
                            $box->clearItems();
                            $box->setWidth(0);
                            $box->setHeight(0);
                            $box->setDepth(0);
                            $box->setWeight(0);
                            $box->setQuantity(0);
                        }

                        $box->setQuantity($box->getQuantity() + 1);

                        if ($item->weight < 1) {
                            $box->setWeight($box->getWeight() + $defaultWeight);
                        } else {
                            $box->setWeight($box->getWeight() + ceil($item->weight));
                        }

                        if ($item->x + $item->y + $item->z < 1) {
                            $box->setWidth($defaultWidth);
                            $box->setHeight($defaultHeight);
                            $box->setDepth($defaultDepth);
                        } else {
                            $sideLength = $box->getLengthSide(ceil($item->x), ceil($item->y), ceil($item->z));
                            $box->setWidth($sideLength);
                            $box->setHeight($sideLength);
                            $box->setDepth($sideLength);
                        }

                        if ($item->quantity - $i == 1 ||
                            $box->getWeight() + ceil($item->weight) > CountriesSettings::LOAD_LIMIT ||
                            $box->getLengthSide(ceil($item->x), ceil($item->y), ceil($item->z)) * 3 > CountriesSettings::MAX_SUM_ALL_SIDES) {
                            if (!$calculate) {
                                $box->addItem($item->id, $item->name, $item->price, $box->getQuantity());
                            }
                            $box->setQuantity(0);
                        }
                    }
                }

                if ($lastItem['PRODUCT_ID'] == $item->id) {
                    $box->addBox($box->getWidth(), $box->getHeight(), $box->getDepth(), $box->getWeight());
                }
            }
        }

        return $box->getBoxSizesArray();
    }

    public static function getAllBoxesWeight($boxesArray)
    {
        $weight = 0;
        foreach ($boxesArray as $box) {
            $weight += $box['Weight'];
        }
        return $weight;
    }
}