<?php

namespace Boxberry\Bitrix\Helpers;

use Bitrix\Main\Page\Asset;
use Boxberry\Bitrix\Handlers\Order;
use CJSCore;

class Extensions
{
    public static function initWidget(): void
    {
        if (!Order::isAdminSection() && !empty(Delivery::getPvzProfilesIds())) {

            if (!CJSCore::IsExtRegistered(Options::getModuleId())) {
                CJSCore::RegisterExt(Options::getModuleId(), [
                    'js' => Options::getJsFolder() . Options::getModuleId() . '/widget.js',
                    'css' => Options::getJsFolder() . Options::getModuleId() . '/widget.css',
                    'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/widget.php',
                    'rel' => [],
                    'use' => CJSCore::USE_PUBLIC,
                    'skip_core' => false,
                ]);
            }

            if (!\CJSCore::isExtensionLoaded(Options::getModuleId())) {
                Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/bb.js?v=' . Options::getModuleVersion());
                \CJSCore::Init([Options::getModuleId()]);
            }

        }
    }

    public static function initOrder(): void
    {
        if (!CJSCore::IsExtRegistered(Options::getModuleId() . '_order')) {
            CJSCore::RegisterExt(Options::getModuleId() . '_order', [
                'js' => Options::getJsFolder() . Options::getModuleId() . '/order.js',
                'css' => Options::getJsFolder() . Options::getModuleId() . '/order.css',
                'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/order.php',
                'rel' => ['date', 'calendar'],
                'skip_core' => false,
                'use' => CJSCore::USE_ADMIN,
            ]);
        }

        if (!\CJSCore::isExtensionLoaded(Options::getModuleId() . '_order')) {
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/bb.js?v=' . Options::getModuleVersion(),);
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/pointsForParcelsListHandler.js?v=' . Options::getModuleVersion());
            \CJSCore::Init([Options::getModuleId() . '_order']);
        }
    }

    public static function initService(): void
    {
        if (!CJSCore::IsExtRegistered(Options::getModuleId() . '_service')) {
            CJSCore::RegisterExt(Options::getModuleId() . '_service', [
                'js' => Options::getJsFolder() . Options::getModuleId() . '/service.js',
                'css' => Options::getJsFolder() . Options::getModuleId() . '/service.css',
                'rel' => ['ui.hint'],
                'use' => CJSCore::USE_ADMIN,
                'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/service.php'
            ]);
        }

        if (!\CJSCore::isExtensionLoaded(Options::getModuleId() . '_service')) {
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/pointsForParcelsListHandler.js?v=' . Options::getModuleVersion());
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/service_helper.js?v=' . Options::getModuleVersion());
            \CJSCore::Init([Options::getModuleId() . '_service']);
        }
    }

    public static function initProfile(): void
    {
        if (!CJSCore::IsExtRegistered(Options::getModuleId() . 'profile')) {
            CJSCore::RegisterExt(Options::getModuleId() . 'profile', [
                'js' => Options::getJsFolder() . Options::getModuleId() . '/profile.js',
                'css' => Options::getJsFolder() . Options::getModuleId() . '/profile.css',
                'rel' => ['ui.hint'],
                'use' => CJSCore::USE_ADMIN,
                'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/profile.php'
            ]);
        }

        if (!\CJSCore::isExtensionLoaded(Options::getModuleId() . 'profile')) {
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/service_helper.js?v=' . Options::getModuleVersion());
            \CJSCore::Init([Options::getModuleId() . 'profile']);
        }
    }

    public static function initOrdersList(): void
    {
        if (!CJSCore::IsExtRegistered(Options::getModuleId() . 'orders_list')) {
            CJSCore::RegisterExt(Options::getModuleId() . 'orders_list', [
                'js' => Options::getJsFolder() . Options::getModuleId() . '/orders_list.js',
                'css' => Options::getJsFolder() . Options::getModuleId() . '/orders_list.css',
                'rel' => [],
                'use' => CJSCore::USE_ADMIN,
                'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/orders_list.php'
            ]);
        }

        if (!\CJSCore::isExtensionLoaded(Options::getModuleId() . 'orders_list')) {
            \CJSCore::Init([Options::getModuleId() . 'orders_list']);
        }
    }

    public static function initSettings(): void
    {
        if (!CJSCore::IsExtRegistered(Options::getModuleId() . 'settings')) {
            CJSCore::RegisterExt(Options::getModuleId() . 'settings', [
                'js' => Options::getJsFolder() . Options::getModuleId() . '/settings.js',
                'css' => Options::getJsFolder() . Options::getModuleId() . '/settings.css',
                'rel' => ['ui.hint'],
                'use' => CJSCore::USE_ADMIN,
                'lang' => '/bitrix/modules/' . Options::getModuleId() . '/lang/' . LANGUAGE_ID . '/extensions/settings.php'
            ]);
        }

        if (!\CJSCore::isExtensionLoaded(Options::getModuleId() . 'settings')) {
            \CJSCore::Init([Options::getModuleId() . 'settings']);
            Asset::getInstance()->addJs(Options::getJsFolder() . Options::getModuleId() . '/options.js?v=' . Options::getModuleVersion());
        }
    }

}