<?php

namespace Boxberry\Bitrix\Helpers;

use Bitrix\Main\ArgumentException;
use Bitrix\Main\Config\Option;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\ObjectPropertyException;
use Bitrix\Main\SystemException;
use Bitrix\Main\Text\StringHelper;
use Bitrix\Sale\Delivery\Services\Table;
use Bitrix\Sale\Internals\BusinessValuePersonDomainTable;
use Bitrix\Sale\Internals\OrderPropsTable;
use Bitrix\Sale\Internals\PaySystemActionTable;
use Bitrix\Sale\Internals\StatusLangTable;
use Boxberry\Api\Entity\ParcelParams;
use CIBlockProperty;
use Exception;

Loc::loadMessages(__FILE__);

class Options
{
    private const MODULE_ID = 'boxberry.delivery';
    private const BB_JS_FOLDER = '/bitrix/js/';
    private const BB_IMG_FOLDER = '/bitrix/images/';
    private const MODULE_VERSION = '1.0.23';
    private const CMS_NAME = 'bitrix';
    public const FIZ_PAYER_TYPE_ID = 'I';
    public const JUR_PAYER_TYPE_ID = 'E';
    private int $deliveryId;
    private string $parentId;

    /**
     * Свойства службы доставки
     */
    private string $apiKey = '';
    private string $testMode = 'N';
    private string $pointForParcelCode = '';
    private string $widgetKey = '';
    private string $issueType = '';
    private string $pointForParcel = '';
    private string $orderPrefix = '';
    private string $defaultWeight = '1000';
    private string $defaultWidth = '20';
    private string $defaultDepth = '30';
    private string $defaultHeight = '10';
    private string $setDefaultsTo = '1';
    private string $zipCheck = 'N';
    private array $codPaySystems = [];
    private string $fioPropType = '1';
    private string $fio = '';
    private string $firstname = '';
    private string $secondname = '';
    private string $middlename = '';
    private string $phone = '';
    private string $email = '';
    private string $addressPropType = '1';
    private string $address = '';
    private string $street = '';
    private string $house = '';
    private string $flat = '';
    private string $location = '';
    private string $zip = '';
    private string $contactPerson = '';
    private string $company = '';
    private string $checkPvzIsSelected = 'Y';
    private string $addressPvzReadonly = 'Y';
    private string $addLinkDefault = 'Y';
    private string $addLinkToDescription = 'N';
    private string $addLinkAfterDescription = 'N';
    private string $addLinkToPeriod = 'N';
    private string $addLinkToCustom = 'N';
    private string $customElementId = '';

    /**
     * Свойства профиля доставки
     */
    private string $profileType = '';
    private string $forcePaymentType = '';
    private string $parselcreateForcePaymentType = '';
    private string $useShopSettings = '1';
    private string $hideDeliveryDay = 'N';
    private string $addDeliveryDay = '0';
    private string $length = '0';
    private string $round = '1';
    private string $rangeFirstCost = '0';
    private string $rangeFirstCalcType = '1';
    private string $rangeFirstFixCost = '0';
    private string $rangeSecondCost = '0';
    private string $rangeSecondCalcType = '1';
    private string $rangeSecondFixCost = '0';
    private string $weight = '0';
    private string $priceBasePercent = '0';
    private string $orderSumPercent = '0';
    private string $paySumPercent = '0';
    private string $fixSum = '0';
    private string $enableMinTariff = 'N';
    private string $minTariff = '0';
    private string $enableMaxTariff = 'N';
    private string $maxTariff = '0';

    /**
     * @throws Exception
     */
    public function __construct($deliveryId)
    {
        if ($parentId = $this->checkParentId($deliveryId)) {
            $this->deliveryId = $deliveryId;
            $this->parentId = $parentId['PARENT_ID'];
            $this->mapParams();
        } else {
            throw new Exception(Loc::getMessage('DELIVERY_IS_NOT_BOXBERRY'));
        }
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    private function checkParentId($deliveryId): bool|array
    {
        return Table::query()
            ->addSelect('PARENT_ID')
            ->where('ID', $deliveryId)
            ->where('CLASS_NAME', Delivery::BB_PROFILE_CLASS_NAME)
            ->setLimit(1)
            ->exec()
            ->fetch();
    }

    public static function getLogoPath(): string
    {
        return self::BB_IMG_FOLDER . self::MODULE_ID . '/bb_logotip.png';
    }

    public static function getLogoId(): bool|int|string
    {
        $logoPath = self::getLogoPath();
        $fileArray = \CFile::MakeFileArray($logoPath);
        return \CFile::SaveFile($fileArray, 'boxberry');
    }

    public function getApiKey(): string
    {
        return $this->apiKey;
    }

    public function setApiKey(string $apiKey): void
    {
        $this->apiKey = $apiKey;
    }

    public function getTestMode(): string
    {
        return $this->testMode;
    }

    public function setTestMode(string $testMode): void
    {
        $this->testMode = $testMode;
    }

    public function getPointForParcelCode(): string
    {
        return $this->pointForParcelCode;
    }

    public function setPointForParcelCode(string $pointForParcelCode): void
    {
        $this->pointForParcelCode = $pointForParcelCode;
    }

    public function getWidgetKey(): string
    {
        return $this->widgetKey;
    }

    public function setWidgetKey(string $widgetKey): void
    {
        $this->widgetKey = $widgetKey;
    }

    public function getIssueType(): string
    {
        return $this->issueType;
    }

    public function setIssueType(string $issueType): void
    {
        $this->issueType = $issueType;
    }

    public function getPointForParcel(): string
    {
        return $this->pointForParcel;
    }

    public function setPointForParcel(string $pointForParcel): void
    {
        $this->pointForParcel = $pointForParcel;
    }

    public function getOrderPrefix(): string
    {
        return $this->orderPrefix;
    }

    public function setOrderPrefix(string $orderPrefix): void
    {
        $this->orderPrefix = $orderPrefix;
    }

    public function getDefaultWeight(): string
    {
        return $this->defaultWeight;
    }

    public function setDefaultWeight(string $defaultWeight): void
    {
        $this->defaultWeight = $defaultWeight;
    }

    public function getDefaultWidth(): string
    {
        return $this->defaultWidth;
    }

    public function setDefaultWidth(string $defaultWidth): void
    {
        $this->defaultWidth = $defaultWidth;
    }

    public function getDefaultDepth(): string
    {
        return $this->defaultDepth;
    }

    public function setDefaultDepth(string $defaultDepth): void
    {
        $this->defaultDepth = $defaultDepth;
    }

    public function getDefaultHeight(): string
    {
        return $this->defaultHeight;
    }

    public function setDefaultHeight(string $defaultHeight): void
    {
        $this->defaultHeight = $defaultHeight;
    }

    public function getSetDefaultsTo(): string
    {
        return $this->setDefaultsTo;
    }

    public function setSetDefaultsTo(string $setDefaultsTo): void
    {
        $this->setDefaultsTo = $setDefaultsTo;
    }

    public function getZipCheck(): bool
    {
        return $this->zipCheck === 'Y';
    }

    public function setZipCheck(string $zipCheck): void
    {
        $this->zipCheck = $zipCheck;
    }

    public function getCodPaySystems(): array
    {
        return $this->codPaySystems;
    }

    public function setCodPaySystems(array $codPaySystems): void
    {
        $this->codPaySystems = $codPaySystems;
    }

    public function getFio(): string
    {
        return $this->fio;
    }

    public function setFio(string $fio): void
    {
        $this->fio = $fio;
    }

    public function getPhone(): string
    {
        return $this->phone;
    }

    public function setPhone(string $phone): void
    {
        $this->phone = $phone;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function setEmail(string $email): void
    {
        $this->email = $email;
    }

    public function getAddress(): string
    {
        return $this->address;
    }

    public function setAddress(string $address): void
    {
        $this->address = $address;
    }

    public function getLocation(): string
    {
        return $this->location;
    }

    public function setLocation(string $location): void
    {
        $this->location = $location;
    }

    public function getZip(): string
    {
        return $this->zip;
    }

    public function setZip(string $zip): void
    {
        $this->zip = $zip;
    }

    public function getContactPerson(): string
    {
        return $this->contactPerson;
    }

    public function setContactPerson(string $contactPerson): void
    {
        $this->contactPerson = $contactPerson;
    }

    public function getCompany(): string
    {
        return $this->company;
    }

    public function setCompany(string $company): void
    {
        $this->company = $company;
    }

    public function getCheckPvzIsSelected(): string
    {
        return $this->checkPvzIsSelected;
    }

    public function setCheckPvzIsSelected(string $checkPvzIsSelected): void
    {
        $this->checkPvzIsSelected = $checkPvzIsSelected;
    }

    public function getAddressPvzReadonly(): string
    {
        return $this->addressPvzReadonly;
    }

    public function setAddressPvzReadonly(string $addressPvzReadonly): void
    {
        $this->addressPvzReadonly = $addressPvzReadonly;
    }

    public function getAddLinkDefault(): string
    {
        return $this->addLinkDefault;
    }

    public function setAddLinkDefault(string $addLinkDefault): void
    {
        $this->addLinkDefault = $addLinkDefault;
    }

    public function getAddLinkToDescription(): string
    {
        return $this->addLinkToDescription;
    }

    public function setAddLinkToDescription(string $addLinkToDescription): void
    {
        $this->addLinkToDescription = $addLinkToDescription;
    }

    public function getAddLinkAfterDescription(): string
    {
        return $this->addLinkAfterDescription;
    }

    public function setAddLinkAfterDescription(string $addLinkAfterDescription): void
    {
        $this->addLinkAfterDescription = $addLinkAfterDescription;
    }

    public function getAddLinkToPeriod(): string
    {
        return $this->addLinkToPeriod;
    }

    public function setAddLinkToPeriod(string $addLinkToPeriod): void
    {
        $this->addLinkToPeriod = $addLinkToPeriod;
    }

    public function getAddLinkToCustom(): string
    {
        return $this->addLinkToCustom;
    }

    public function setAddLinkToCustom(string $addLinkToCustom): void
    {
        $this->addLinkToCustom = $addLinkToCustom;
    }

    public function getCustomElementId(): string
    {
        return trim($this->customElementId);
    }

    public function setCustomElementId(string $customElementId): void
    {
        $this->customElementId = $customElementId;
    }

    public function isAddLinkToCustom(): bool
    {
        return $this->addLinkToCustom === 'Y';
    }

    public function getProfileType(): string
    {
        return $this->profileType;
    }

    public function setProfileType(string $profileType): void
    {
        $this->profileType = $profileType;
    }

    public function isCalcSourceProfile(): bool
    {
        return $this->useShopSettings === '2';
    }

    public function isEnableMinTariff(): bool
    {
        return $this->enableMinTariff === 'Y';
    }

    public function isEnableMaxTariff(): bool
    {
        return $this->enableMaxTariff === 'Y';
    }

    public function isHideDeliveryDays(): bool
    {
        return $this->hideDeliveryDay === 'Y';
    }

    public function getUseShopSettings(): string
    {
        return $this->useShopSettings;
    }

    public function setUseShopSettings(string $useShopSettings): void
    {
        $this->useShopSettings = $useShopSettings;
    }

    public function getHideDeliveryDay(): string
    {
        return $this->hideDeliveryDay;
    }

    public function setHideDeliveryDay(string $hideDeliveryDay): void
    {
        $this->hideDeliveryDay = $hideDeliveryDay;
    }

    public function getAddDeliveryDay(): string
    {
        return $this->addDeliveryDay;
    }

    public function setAddDeliveryDay(string $addDeliveryDay): void
    {
        $this->addDeliveryDay = $addDeliveryDay;
    }

    public function getLength(): string
    {
        return $this->length;
    }

    public function setLength(string $length): void
    {
        $this->length = $length;
    }

    public function getRound(): string
    {
        return $this->round;
    }

    public function setRound(string $round): void
    {
        $this->round = $round;
    }

    public function getRangeFirstCost(): string
    {
        return $this->rangeFirstCost;
    }

    public function setRangeFirstCost(string $rangeFirstCost): void
    {
        $this->rangeFirstCost = $rangeFirstCost;
    }

    public function getRangeFirstCalcType(): string
    {
        return $this->rangeFirstCalcType;
    }

    public function setRangeFirstCalcType(string $rangeFirstCalcType): void
    {
        $this->rangeFirstCalcType = $rangeFirstCalcType;
    }

    public function getRangeFirstFixCost(): string
    {
        return $this->rangeFirstFixCost;
    }

    public function setRangeFirstFixCost(string $rangeFirstFixCost): void
    {
        $this->rangeFirstFixCost = $rangeFirstFixCost;
    }

    public function getRangeSecondCost(): string
    {
        return $this->rangeSecondCost;
    }

    public function setRangeSecondCost(string $rangeSecondCost): void
    {
        $this->rangeSecondCost = $rangeSecondCost;
    }

    public function getRangeSecondCalcType(): string
    {
        return $this->rangeSecondCalcType;
    }

    public function setRangeSecondCalcType(string $rangeSecondCalcType): void
    {
        $this->rangeSecondCalcType = $rangeSecondCalcType;
    }

    public function getRangeSecondFixCost(): string
    {
        return $this->rangeSecondFixCost;
    }

    public function setRangeSecondFixCost(string $rangeSecondFixCost): void
    {
        $this->rangeSecondFixCost = $rangeSecondFixCost;
    }

    public function getWeight(): string
    {
        return $this->weight;
    }

    public function setWeight(string $weight): void
    {
        $this->weight = $weight;
    }

    public function getPriceBasePercent(): string
    {
        return $this->priceBasePercent;
    }

    public function setPriceBasePercent(string $priceBasePercent): void
    {
        $this->priceBasePercent = $priceBasePercent;
    }

    public function getOrderSumPercent(): string
    {
        return $this->orderSumPercent;
    }

    public function setOrderSumPercent(string $orderSumPercent): void
    {
        $this->orderSumPercent = $orderSumPercent;
    }

    public function getPaySumPercent(): string
    {
        return $this->paySumPercent;
    }

    public function setPaySumPercent(string $paySumPercent): void
    {
        $this->paySumPercent = $paySumPercent;
    }

    public function getFixSum(): string
    {
        return $this->fixSum;
    }

    public function setFixSum(string $fixSum): void
    {
        $this->fixSum = $fixSum;
    }

    public function getEnableMinTariff(): string
    {
        return $this->enableMinTariff;
    }

    public function setEnableMinTariff(string $enableMinTariff): void
    {
        $this->enableMinTariff = $enableMinTariff;
    }

    public function getMinTariff(): string
    {
        return $this->minTariff;
    }

    public function setMinTariff(string $minTariff): void
    {
        $this->minTariff = $minTariff;
    }

    public function getEnableMaxTariff(): string
    {
        return $this->enableMaxTariff;
    }

    public function setEnableMaxTariff(string $enableMaxTariff): void
    {
        $this->enableMaxTariff = $enableMaxTariff;
    }

    public function getMaxTariff(): string
    {
        return $this->maxTariff;
    }

    public function setMaxTariff(string $maxTariff): void
    {
        $this->maxTariff = $maxTariff;
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    private function getParams(): array
    {
        return Table::query()
            ->addSelect('CONFIG')
            ->whereIn('ID', [$this->parentId, $this->deliveryId])
            ->setLimit(2)
            ->setCacheTtl(3600)
            ->exec()
            ->fetchAll();
    }

    /**
     * @throws ArgumentException
     * @throws ObjectPropertyException
     * @throws SystemException
     */
    private function mapParams(): void
    {
        $params = $this->getParams();

        foreach ($params as $param) {
            if (!isset($param['CONFIG'])) {
                continue;
            }

            foreach ($param['CONFIG'] as $groupName => $groupParams) {
                if ($groupName === 'COD_PAY_SYSTEMS' && is_array($groupParams)) {
                    $this->codPaySystems = $groupParams;
                    continue;
                }

                foreach ($groupParams as $key => $value) {
                    $key = lcfirst(StringHelper::snake2camel($key));

                    if (!property_exists($this, $key)) {
                        continue;
                    }

                    $this->$key = $value;
                }
            }
        }
    }

    public function isTestMode(): bool
    {
        return $this->testMode === 'Y';
    }

    public function isPaySystemCod($currentPaySystemId): bool
    {
        return isset($this->codPaySystems[$currentPaySystemId]) && $this->codPaySystems[$currentPaySystemId] === 'Y';
    }

    public function isAddLinkToDescription(): bool
    {
        return $this->addLinkToDescription === 'Y';
    }

    public function isAddLinkAfterDescription(): bool
    {
        return $this->addLinkAfterDescription === 'Y';
    }

    public function isAddLinkToPeriod(): bool
    {
        return $this->addLinkToPeriod === 'Y';
    }

    public function isAddLinkDefault(): bool
    {
        return $this->addLinkDefault === 'Y';
    }

    public function isCourier(): bool
    {
        return $this->profileType === '2';
    }

    public function isPickup(): bool
    {
        return $this->profileType === '1';
    }

    public function isSetDefaultsForOrder(): bool
    {
        return $this->setDefaultsTo === '1';
    }

    public function isCheckPvzIsSelected(): bool
    {
        return $this->checkPvzIsSelected === 'Y';
    }

    public function isAddressPvzReadOnly(): bool
    {
        return $this->addressPvzReadonly === 'Y';
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    public static function getActivePaymentMethods(): array
    {
        $paymentMethods = PaySystemActionTable::query()
            ->where('ACTIVE', 'Y')
            ->setSelect(['ID', 'NAME'])
            ->exec()
            ->fetchAll();

        if (empty($paymentMethods)) {
            return [];
        }

        return array_map(function ($method) {
            return [
                'TYPE' => 'Y/N',
                'NAME' => $method['NAME'],
                'REFRESH' => 'Y',
                'DEFAULT' => 'N',
            ];
        }, array_column($paymentMethods, null, 'ID'));
    }

    public static function getBbStatusesArray(): array
    {
        return [
            '0' => Loc::getMessage('STATUS_NAME_0'),
            '1' => Loc::getMessage('STATUS_NAME_1'),
            '5' => Loc::getMessage('STATUS_NAME_5'),
            '7' => Loc::getMessage('STATUS_NAME_7'),
            '10' => Loc::getMessage('STATUS_NAME_10'),
            '76' => Loc::getMessage('STATUS_NAME_76'),
            '77' => Loc::getMessage('STATUS_NAME_77'),
            '80' => Loc::getMessage('STATUS_NAME_80'),
            '90' => Loc::getMessage('STATUS_NAME_90'),
            '100' => Loc::getMessage('STATUS_NAME_100'),
            '110' => Loc::getMessage('STATUS_NAME_110'),
            '115' => Loc::getMessage('STATUS_NAME_115'),
            '120' => Loc::getMessage('STATUS_NAME_120'),
            '125' => Loc::getMessage('STATUS_NAME_125'),
            '130' => Loc::getMessage('STATUS_NAME_130'),
            '135' => Loc::getMessage('STATUS_NAME_135'),
            '137' => Loc::getMessage('STATUS_NAME_137'),
            '140' => Loc::getMessage('STATUS_NAME_140'),
            '150' => Loc::getMessage('STATUS_NAME_150'),
            '160' => Loc::getMessage('STATUS_NAME_160'),
            '170' => Loc::getMessage('STATUS_NAME_170'),
            '180' => Loc::getMessage('STATUS_NAME_180'),
            '190' => Loc::getMessage('STATUS_NAME_190'),
        ];
    }

    public static function getBitrixOrderStatuses(): array
    {
        try {
            $results = StatusLangTable::query()
                ->setOrder(['STATUS.SORT' => 'ASC', 'STATUS_ID' => 'ASC'])
                ->setSelect(['STATUS_ID', 'NAME'])
                ->where('STATUS.TYPE', 'O')
                ->where('LID', 'RU')
                ->exec()
                ->fetchAll();
        } catch (Exception $e) {
            return [];
        }

        $bitrixStatuses = [];

        foreach ($results as $result) {
            $bitrixStatuses[$result['STATUS_ID']] = $result['NAME'];
        }

        return $bitrixStatuses;
    }

    public static function getBitrixShipmentStatuses(): array
    {
        try {
            $results = StatusLangTable::query()
                ->setOrder(['STATUS.SORT' => 'ASC', 'STATUS_ID' => 'ASC'])
                ->setSelect(['STATUS_ID', 'NAME'])
                ->where('STATUS.TYPE', 'D')
                ->where('LID', 'RU')
                ->exec()
                ->fetchAll();
        } catch (Exception $e) {
            return [];
        }

        $bitrixStatuses = [];

        foreach ($results as $result) {
            $bitrixStatuses[$result['STATUS_ID']] = $result['NAME'];
        }

        return $bitrixStatuses;
    }

    public static function getOrderProps(): array
    {
        $bbProps = self::getBbProps();
        $options = [];
        foreach ($bbProps as $key => $value) {
            $options[$key] = [
                'TYPE' => 'STRING',
                'NAME' => $value,
                'REFRESH' => 'Y',
                'DEFAULT' => $key,
                'ID' => 'bb' . StringHelper::snake2camel($key)
            ];
        }

        return $options;
    }

    public static function getBitrixPropsCodes(): array
    {
        try {
            $result = OrderPropsTable::query()
                ->addSelect('CODE')
                ->where('ACTIVE', 'Y')
                ->exec()
                ->fetchAll();

            if (empty($result)) {
                return [];
            }

            return array_column($result, 'CODE');
        } catch (Exception $e) {
            return [];
        }
    }

    public static function getBbProps(): array
    {
        return [
            'FIO' => Loc::getMessage('FIO_PROP_NAME'),
            'FIRSTNAME' => Loc::getMessage('FIRSTNAME_PROP_NAME'),
            'SECONDNAME' => Loc::getMessage('SECONDNAME_PROP_NAME'),
            'MIDDLENAME' => Loc::getMessage('MIDDLENAME_PROP_NAME'),
            'PHONE' => Loc::getMessage('PHONE_PROP_NAME'),
            'EMAIL' => Loc::getMessage('EMAIL_PROP_NAME'),
            'ADDRESS' => Loc::getMessage('ADDRESS_PROP_NAME'),
            'STREET' => Loc::getMessage('STREET_PROP_NAME'),
            'HOUSE' => Loc::getMessage('HOUSE_PROP_NAME'),
            'FLAT' => Loc::getMessage('FLAT_PROP_NAME'),
            'LOCATION' => Loc::getMessage('LOCATION_PROP_NAME'),
            'ZIP' => Loc::getMessage('ZIP_PROP_NAME'),
            'CONTACT_PERSON' => Loc::getMessage('CONTACT_PERSON_PROP_NAME'),
            'COMPANY' => Loc::getMessage('COMPANY_PROP_NAME'),
        ];
    }

    public static function isProductPropExist($propertyCode): bool
    {
        $dbRes = CIBlockProperty::GetList([], ['CODE' => $propertyCode]);

        return (bool)(($dbRes->Fetch()));
    }

    public static function getPayerTypeId($personTypeId): string
    {
        try {
            $payerType = BusinessValuePersonDomainTable::query()
                ->addSelect('DOMAIN')
                ->where('PERSON_TYPE_ID', $personTypeId)
                ->setLimit(1)
                ->exec()
                ->fetch();

            if (isset($payerType['DOMAIN'])) {
                return $payerType['DOMAIN'];
            }

        } catch (Exception $e) {
        }

        return '';
    }

    public static function getIssueTypesArray(): array
    {
        return [
            '' => Loc::getMessage('ISSUE_TYPE_LIKE_LK'),
            ParcelParams::ISSUE_TYPE_WITHOUT_OPENING_ID => Loc::getMessage('ISSUE_TYPE_WITHOUT_OPENING'),
            ParcelParams::ISSUE_TYPE_WITH_OPENING_AND_VERIFICATION_ID => Loc::getMessage('ISSUE_TYPE_WITH_OPENING_AND_VERIFICATION'),
            ParcelParams::ISSUE_TYPE_PARTIAL_ID => Loc::getMessage('ISSUE_TYPE_PARTIAL'),
        ];
    }

    /**
     * @throws ObjectPropertyException
     * @throws ArgumentException
     * @throws SystemException
     */
    public static function getConfigStructureForHandler(): array
    {
        return [
            'MAIN' => [
                'TITLE' => Loc::getMessage('HANDLER_MAIN_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_MAIN_DESC'),
                'ITEMS' => [
                    'API_KEY' => [
                        'TYPE' => 'STRING',
                        'SIZE' => 30,
                        'NAME' => Loc::getMessage('API_KEY_NAME'),
                        'DESCRIPTION' => Loc::getMessage('API_KEY_DESC'),
                        'ID' => 'bbApiToken'
                    ],
                    'TEST_MODE' => [
                        'TYPE' => 'Y/N',
                        'NAME' => Loc::getMessage('TEST_MODE_NAME'),
                        'DEFAULT' => 'N',
                        'ID' => 'bbTestMode'
                    ],
                    'POINT_FOR_PARCEL_CODE' => [
                        'TYPE' => 'STRING',
                        'HIDDEN' => 'Y',
                        'DEFAULT' => '',
                        'NAME' => Loc::getMessage('POINT_FOR_PARCEL_CODE_NAME'),
                        'ID' => 'bbPointForParcelCode'
                    ],
                    'WIDGET_KEY' => [
                        'TYPE' => 'STRING',
                        'HIDDEN' => 'Y',
                        'NAME' => Loc::getMessage('WIDGET_KEY_NAME'),
                        'ID' => 'bbWidgetKey'
                    ],
                ]
            ],
            'ORDERS' => [
                'TITLE' => Loc::getMessage('HANDLER_ORDERS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_ORDERS_DESC'),
                'ITEMS' => [
                    'ISSUE_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('ISSUE_TYPE_NAME'),
                        'DEFAULT' => '',
                        'OPTIONS' => self::getIssueTypesArray(),
                        'ID' => 'bbIssueType'
                    ],
                    'POINT_FOR_PARCEL' => [
                        'TYPE' => 'STRING',
                        'DEFAULT' => '',
                        'NAME' => Loc::getMessage('POINT_FOR_PARCEL_NAME'),
                        'ID' => 'bbPointForParcel'
                    ],
                    'ORDER_PREFIX' => [
                        'TYPE' => 'STRING',
                        'NAME' => Loc::getMessage('ORDER_PREFIX_NAME'),
                        'ID' => 'bbOrderPrefix'
                    ],
                ]
            ],
            'WEIGHTS' => [
                'TITLE' => Loc::getMessage('HANDLER_WEIGHTS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_WEIGHTS_DESC'),
                'ITEMS' => [
                    'DEFAULT_WEIGHT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('DEFAULT_WEIGHT_NAME'),
                        'REQUIRED' => 'Y',
                        'DEFAULT' => '1000',
                        'ID' => 'bbDefaultWeight'
                    ],
                    'DEFAULT_WIDTH' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('DEFAULT_WIDTH_NAME'),
                        'REQUIRED' => 'Y',
                        'DEFAULT' => '20',
                        'ID' => 'bbDefaultWidth'
                    ],
                    'DEFAULT_DEPTH' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('DEFAULT_DEPTH_NAME'),
                        'REQUIRED' => 'Y',
                        'DEFAULT' => '30',
                        'ID' => 'bbDefaultDepth'
                    ],
                    'DEFAULT_HEIGHT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('DEFAULT_HEIGHT_NAME'),
                        'REQUIRED' => 'Y',
                        'DEFAULT' => '10',
                        'ID' => 'bbDefaultHeight'
                    ],
                    'SET_DEFAULTS_TO' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('SET_DEFAULTS_TO_NAME'),
                        'DEFAULT' => '1',
                        'OPTIONS' => [
                            '1' => Loc::getMessage('SET_DEFAULTS_TO_OPTION_1'),
                            '2' => Loc::getMessage('SET_DEFAULTS_TO_OPTION_2'),
                        ],
                        'ID' => 'bbSetForOrder'
                    ],
                ]
            ],
            'ZIPS' => [
                'TITLE' => Loc::getMessage('HANDLER_ZIPS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_ZIPS_DESC'),
                'ITEMS' => [
                    'ZIP_CHECK' => [
                        'TYPE' => 'Y/N',
                        'NAME' => Loc::getMessage('ZIP_CHECK_NAME'),
                        'DEFAULT' => 'N',
                        'ID' => 'bbZipCheck'
                    ],
                ]
            ],
            'COD_PAY_SYSTEMS' => [
                'TITLE' => Loc::getMessage('HANDLER_COD_PAY_SYSTEMS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_COD_PAY_SYSTEMS_DESC'),
                'ITEMS' => self::getActivePaymentMethods(),
                'ID' => 'bbCodPaysystems'
            ],
            'USER_PROPS' => [
                'TITLE' => Loc::getMessage('HANDLER_USER_PROPS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('HANDLER_USER_PROPS_DESC'),
                'ITEMS' => [
                        'FIO_PROP_TYPE' => [
                            'TYPE' => 'ENUM',
                            'NAME' => Loc::getMessage('FIO_PROP_TYPE_NAME'),
                            'DEFAULT' => '1',
                            'OPTIONS' => [
                                '1' => Loc::getMessage('FIO_PROP_TYPE_OPTION_1'),
                                '2' => Loc::getMessage('FIO_PROP_TYPE_OPTION_2'),
                            ],
                            'ID' => 'bbFioPropType'
                        ],
                        'ADDRESS_PROP_TYPE' => [
                            'TYPE' => 'ENUM',
                            'NAME' => Loc::getMessage('ADDRESS_PROP_TYPE_NAME'),
                            'DEFAULT' => '1',
                            'OPTIONS' => [
                                '1' => Loc::getMessage('ADDRESS_PROP_TYPE_OPTION_1'),
                                '2' => Loc::getMessage('ADDRESS_PROP_TYPE_OPTION_2'),
                            ],
                            'ID' => 'bbAddressPropType'
                        ],

                    ] + self::getOrderProps(),
            ],
            'DELIVERY_PROCCESS' =>
                [
                    'TITLE' => Loc::getMessage('HANDLER_DELIVERY_PROCCESS_TITLE'),
                    'DESCRIPTION' => Loc::getMessage('HANDLER_DELIVERY_PROCCESS_DESC'),
                    'ITEMS' => [
                        'CHECK_PVZ_IS_SELECTED' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('CHECK_PVZ_IS_SELECTED_NAME'),
                            'DEFAULT' => 'Y',
                            'ID' => 'bbCheckPvzIsSelected'
                        ],
                        'ADDRESS_PVZ_READONLY' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADDRESS_PVZ_READONLY_NAME'),
                            'DEFAULT' => 'Y',
                            'ID' => 'bbAddressPvzReadOnly'
                        ],
                        'ADD_LINK_DEFAULT' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADD_LINK_DEFAULT_NAME'),
                            'DEFAULT' => 'Y',
                            'ID' => 'bbAddLinkDefault'
                        ],
                        'ADD_LINK_TO_DESCRIPTION' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADD_LINK_TO_DESCRIPTION_NAME'),
                            'DEFAULT' => 'N',
                            'ID' => 'bbAddLinkToDescription'
                        ],
                        'ADD_LINK_AFTER_DESCRIPTION' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADD_LINK_AFTER_DESCRIPTION_NAME'),
                            'DEFAULT' => 'N',
                            'ID' => 'bbAddLinkAfterDescription'
                        ],
                        'ADD_LINK_TO_PERIOD' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADD_LINK_TO_PERIOD_NAME'),
                            'DEFAULT' => 'N',
                            'ID' => 'bbAddLinkToPeriod'
                        ],
                        'ADD_LINK_TO_CUSTOM' => [
                            'TYPE' => 'Y/N',
                            'NAME' => Loc::getMessage('ADD_LINK_TO_CUSTOM_NAME'),
                            'DEFAULT' => 'N',
                            'ID' => 'bbAddLinkToCustom'
                        ],
                        'CUSTOM_ELEMENT_ID' => [
                            'TYPE' => 'STRING',
                            'NAME' => Loc::getMessage('CUSTOM_ELEMENT_ID_NAME'),
                            'ID' => 'bbCustomElementId'
                        ],
                    ]
                ],
        ];
    }

    public static function getConfigStructureForProfile(): array
    {
        return [
            'TYPE' => [
                'TITLE' => Loc::getMessage('PROFILE_TYPE_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_TYPE_DESC'),
                'ITEMS' => [
                    'PROFILE_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('PROFILE_TYPE_NAME'),
                        'DEFAULT' => '',
                        'REQUIRED' => 'Y',
                        'ID' => 'bbProfileType',
                        'OPTIONS' => [
                            '' => Loc::getMessage('STATUS_NAME_0'),
                            '1' => Loc::getMessage('PROFILE_TYPE_OPTION_1'),
                            '2' => Loc::getMessage('PROFILE_TYPE_OPTION_2'),
                        ]
                    ],
                ]
            ],
            'FORCE_PAYMENT_TYPES' => [
                'TITLE' => Loc::getMessage('PROFILE_FORCE_PAYMENT_TYPES_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_FORCE_PAYMENT_TYPES_DESC'),
                'TYPE' => 'ENUM',
                'DEFAULT' => '',
                'ITEMS' => [
                    'FORCE_PAYMENT_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('FORCE_PAYMENT_TYPE_NAME'),
                        'DEFAULT' => '',
                        'ID' => 'bbForcePaymentType',
                        'OPTIONS' => [
                            '' => Loc::getMessage('FORCE_PAYMENT_TYPE_OPTION_0'),
                            '1' => Loc::getMessage('FORCE_PAYMENT_TYPE_OPTION_1'),
                            '2' => Loc::getMessage('FORCE_PAYMENT_TYPE_OPTION_2'),
                        ],
                    ],
                    'PARSELCREATE_FORCE_PAYMENT_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('PARSELCREATE_FORCE_PAYMENT_TYPE_NAME'),
                        'DEFAULT' => '',
                        'ID' => 'bbParselcreateForcePaymentType',
                        'OPTIONS' => [
                            '' => Loc::getMessage('PARSELCREATE_FORCE_PAYMENT_TYPE_OPTION_0'),
                            '1' => Loc::getMessage('PARSELCREATE_FORCE_PAYMENT_TYPE_OPTION_1'),
                            '2' => Loc::getMessage('PARSELCREATE_FORCE_PAYMENT_TYPE_OPTION_2'),
                        ],
                    ],
                ],
            ],
            'CALC_SOURCE' => [
                'TITLE' => Loc::getMessage('PROFILE_CALC_SOURCE_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_CALC_SOURCE_DESC'),
                'ITEMS' => [
                    'USE_SHOP_SETTINGS' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('USE_SHOP_SETTINGS_NAME'),
                        'DEFAULT' => '1',
                        'ID' => 'bbUseShopSettings',
                        'OPTIONS' => [
                            '1' => Loc::getMessage('USE_SHOP_SETTINGS_OPTION_1'),
                            '2' => Loc::getMessage('USE_SHOP_SETTINGS_OPTION_2'),
                        ]
                    ]
                ]
            ],
            'DELIVERY_PERIOD' => [
                'TITLE' => Loc::getMessage('PROFILE_DELIVERY_PERIOD_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_DELIVERY_PERIOD_DESC'),
                'ITEMS' => [
                    'HIDE_DELIVERY_DAY' => [
                        'TYPE' => 'Y/N',
                        'NAME' => Loc::getMessage('HIDE_DELIVERY_DAY_NAME'),
                        'DEFAULT' => 'N',
                        'ID' => 'bbHideDeliveryDay'
                    ],
                    'ADD_DELIVERY_DAY' => [
                        'TYPE' => 'NUMBER',
                        'SIZE' => 20,
                        'NAME' => Loc::getMessage('ADD_DELIVERY_DAY_NAME'),
                        'DEFAULT' => 0,
                        'ID' => 'bbAddDeliveryDay'
                    ],
                ]
            ],
            'LENGTH_AND_ROUND' => [
                'TITLE' => Loc::getMessage('PROFILE_LENGTH_AND_ROUND_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_LENGTH_AND_ROUND_DESC'),
                'ITEMS' => [
                    'LENGTH' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('LENGTH_NAME'),
                        'DEFAULT' => '0',
                        'ID' => 'bbLength',
                        'OPTIONS' => [
                            '0' => Loc::getMessage('LENGTH_OPTION_0'),
                            '1' => Loc::getMessage('LENGTH_OPTION_1'),
                            '2' => Loc::getMessage('LENGTH_OPTION_2'),
                            '3' => Loc::getMessage('LENGTH_OPTION_3'),
                        ]
                    ],
                    'ROUND' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('ROUND_NAME'),
                        'DEFAULT' => '1',
                        'ID' => 'bbRound',
                        'OPTIONS' => [
                            '1' => Loc::getMessage('ROUND_OPTION_1'),
                            '2' => Loc::getMessage('ROUND_OPTION_2'),
                        ]
                    ]
                ]
            ],
            'COSTS' => [
                'TITLE' => Loc::getMessage('PROFILE_COSTS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_COSTS_DESC'),
                'ITEMS' => [
                    'RANGE_FIRST_COST' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('RANGE_FIRST_COST_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbRange1Cost'
                    ],
                    'RANGE_FIRST_CALC_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('RANGE_FIRST_CALC_TYPE_NAME'),
                        'DEFAULT' => '1',
                        'ID' => 'bbRange1CalcType',
                        'OPTIONS' => [
                            '1' => Loc::getMessage('RANGE_FIRST_CALC_TYPE_OPTION_1'),
                            '2' => Loc::getMessage('RANGE_FIRST_CALC_TYPE_OPTION_2'),
                        ]
                    ],
                    'RANGE_FIRST_FIX_COST' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('RANGE_FIRST_FIX_COST_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbRange1FixCost'
                    ],
                    'RANGE_SECOND_COST' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('RANGE_SECOND_COST_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbRange2Cost'
                    ],
                    'RANGE_SECOND_CALC_TYPE' => [
                        'TYPE' => 'ENUM',
                        'NAME' => Loc::getMessage('RANGE_SECOND_CALC_TYPE_NAME'),
                        'DEFAULT' => '1',
                        'ID' => 'bbRange2CalcType',
                        'OPTIONS' => [
                            '1' => Loc::getMessage('RANGE_SECOND_CALC_TYPE_OPTION_1'),
                            '2' => Loc::getMessage('RANGE_SECOND_CALC_TYPE_OPTION_2'),
                        ]
                    ],
                    'RANGE_SECOND_FIX_COST' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('RANGE_SECOND_FIX_COST_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbRange2FixCost'
                    ],

                ]
            ],
            'SURCHARGE' => [
                'TITLE' => Loc::getMessage('PROFILE_SURCHARGE_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_SURCHARGE_DESC'),
                'ITEMS' => [
                    'WEIGHT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('WEIGHT_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbWeight'
                    ],
                    'PRICE_BASE_PERCENT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('PRICE_BASE_PERCENT_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbPriceBasePercent'
                    ],
                    'ORDER_SUM_PERCENT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('ORDER_SUM_PERCENT_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbOrderSumPercent'
                    ],
                    'PAY_SUM_PERCENT' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('PAY_SUM_PERCENT_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbPaySumPercent'
                    ],
                    'FIX_SUM' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('FIX_SUM_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbFixSum'
                    ],
                ]
            ],
            'OPTIONS' => [
                'TITLE' => Loc::getMessage('PROFILE_OPTIONS_TITLE'),
                'DESCRIPTION' => Loc::getMessage('PROFILE_OPTIONS_DESC'),
                'ITEMS' => [
                    'ENABLE_MIN_TARIFF' => [
                        'TYPE' => 'Y/N',
                        'NAME' => Loc::getMessage('ENABLE_MIN_TARIFF_NAME'),
                        'DEFAULT' => 'N',
                        'ID' => 'bbEnableMinTariff'
                    ],
                    'MIN_TARIFF' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('MIN_TARIFF_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbMinTariff'
                    ],
                    'ENABLE_MAX_TARIFF' => [
                        'TYPE' => 'Y/N',
                        'NAME' => Loc::getMessage('ENABLE_MAX_TARIFF_NAME'),
                        'DEFAULT' => 'N',
                        'ID' => 'bbEnableMaxTariff'
                    ],
                    'MAX_TARIFF' => [
                        'TYPE' => 'NUMBER',
                        'NAME' => Loc::getMessage('MAX_TARIFF_NAME'),
                        'SIZE' => 20,
                        'DEFAULT' => 0,
                        'ID' => 'bbMaxTariff'
                    ],
                ]
            ]

        ];
    }

    public static function getModuleVersion(): string
    {
        return self::MODULE_VERSION;
    }

    public static function getModuleId(): string
    {
        return self::MODULE_ID;
    }

    public static function getCmsName(): string
    {
        return self::CMS_NAME;
    }

    public static function getJsFolder(): string
    {
        return self::BB_JS_FOLDER;
    }

    public static function getImgFolder(): string
    {
        return self::BB_IMG_FOLDER;
    }

    public function getParentId(): string
    {
        return $this->parentId;
    }

    public function setParentId(string $parentId): void
    {
        $this->parentId = $parentId;
    }

    public function getDeliveryId(): int
    {
        return $this->deliveryId;
    }

    public function setDeliveryId(int $deliveryId): void
    {
        $this->deliveryId = $deliveryId;
    }

    public static function getStatusMappings(): array
    {
        $bitrixStatuses = array_keys(self::getBitrixOrderStatuses());
        $statusesOptions = [];

        foreach ($bitrixStatuses as $bitrixStatus) {
            if ($bbStatus = Option::get(self::MODULE_ID, $bitrixStatus)) {
                $statusesOptions[$bitrixStatus] = $bbStatus;
            }
        }

        return $statusesOptions;

    }

    public static function getShipmentStatusMappings(): array
    {
        $bitrixStatuses = array_keys(self::getBitrixShipmentStatuses());
        $statusesOptions = [];

        foreach ($bitrixStatuses as $bitrixStatus) {
            if ($bbStatus = Option::get(self::MODULE_ID, $bitrixStatus)) {
                $statusesOptions[$bitrixStatus] = $bbStatus;
            }
        }

        return $statusesOptions;
    }

    public static function isNeedShippedFlag($shipmentStatusId): bool
    {
        return Option::get(self::MODULE_ID, $shipmentStatusId . '_shipped') === 'Y';
    }

    public static function getShippedFlagByStatusId($shipmentStatusId): string
    {
        return Option::get(self::MODULE_ID, $shipmentStatusId . '_shipped', 'N');
    }

    public static function isStatusExist($statusId): bool
    {
        try {
            $status = StatusLangTable::query()
                ->where('STATUS_ID', $statusId)
                ->where('LID', 'ru')
                ->setLimit(1)
                ->exec()
                ->fetch();

            return !empty($status);
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @throws ObjectPropertyException
     * @throws SystemException
     * @throws ArgumentException
     */
    private static function getApiTokenByParentId($id)
    {
        $result = Table::query()
            ->addSelect('CONFIG')
            ->where('ID', $id)
            ->setLimit(1)
            ->exec()
            ->fetch();

        return $result['CONFIG']['MAIN']['API_KEY'];
    }

    public static function getApiTokens(): array
    {
        $tokens = [];
        $handlerIds = Delivery::getBbHandlersIds();

        foreach ($handlerIds as $handlerId) {
            try {
                $tokens[$handlerId] = self::getApiTokenByParentId($handlerId);
            } catch (Exception $e) {
                continue;
            }

        }

        return $tokens;
    }

    public static function getBitrixStatusIdByBbStatusId($bbStatusId): string
    {
        $statusesOptions = self::getStatusMappings();

        foreach ($statusesOptions as $bitrixStatusId => $bbStatus) {
            if ($bbStatus == $bbStatusId) {
                return $bitrixStatusId;
            }
        }

        return '';
    }

    public static function getShipmentStatusIdByBbStatusId($bbStatusId): string
    {
        $statusesOptions = self::getShipmentStatusMappings();

        foreach ($statusesOptions as $bitrixStatusId => $bbStatus) {
            if ($bbStatus == $bbStatusId) {
                return $bitrixStatusId;
            }
        }

        return '';
    }

    public static function isEventLogEnabled(): bool
    {
        return Option::get(self::getModuleId(), "EVENT_LOG_ENABLE", "N") === 'Y';
    }

    public static function isApiLogEnabled(): bool
    {
        return Option::get(self::getModuleId(), "API_LOG_ENABLE", "N") === 'Y';
    }

    public static function getApiLogFilePath(): string
    {
        return Option::get(self::getModuleId(), "API_LOG_FILE_PATH", '/bitrix/modules/boxberry.delivery/api.log');
    }

    public static function getApiLogFileSize(): int
    {
        return (int)Option::get(self::getModuleId(), "API_LOG_FILE_SIZE", '2');
    }

    public static function getArtNumberPropCode(): string
    {
        return Option::get(self::getModuleId(), "ARTNUMBER_PROP_CODE", 'ARTNUMBER');
    }

    public static function isEnabledCreateOrderAdminForAll(): bool
    {
        return Option::get(self::getModuleId(), "CREATE_ORDER_ADMIN_FOR_ALL", 'Y') === 'Y';
    }

    public static function getOrderStatusForExport(): string
    {
        return Option::get(self::getModuleId(), "ORDER_STATUS_FOR_EXPORT", '');
    }

    public static function isEnabledSetTrackToShipment(): bool
    {
        return Option::get(self::getModuleId(), "SET_TRACK_TO_SHIPMENT", 'Y') === 'Y';
    }

    public static function isEnabledInitWidgetOnlyOnStandardEvent(): bool
    {
        return Option::get(self::getModuleId(), "INIT_WIDGET_ONLY_ON_STANDARD_EVENT", 'N') === 'Y';
    }

    public static function isConsoleLogEnabled(): bool
    {
        return Option::get(self::getModuleId(), "CONSOLE_LOG_ENABLE", 'N') === 'Y';
    }

    public static function isStatusSyncEnabled(): bool
    {
        return Option::get(self::getModuleId(), "STATUS_SYNC_ENABLE", "N") === 'Y';
    }

    public static function getStatusSyncOrdersLimit(): string
    {
        return Option::get(self::getModuleId(), "STATUS_SYNC_ORDERS_LIMIT", 100);
    }

    public static function getStatusSyncOrdersExcludeDays(): string
    {
        return Option::get(self::getModuleId(), "STATUS_SYNC_ORDERS_EXCLUDE_DAYS", 60);
    }

    public static function isSetOrderPaidWhenDelivered(): bool
    {
        return Option::get(self::getModuleId(), "SET_ORDER_PAID_WHEN_DELIVERED", "N") === 'Y';
    }

    public static function isEnableDadataUserToken(): bool
    {
        return Option::get(self::getModuleId(), "ENABLE_DADATA_USER_TOKEN", "N") === 'Y';
    }

    public static function getDadataUserToken(): string
    {
        return Option::get(self::getModuleId(), "DADATA_USER_TOKEN", "");
    }

    public static function getGeoIdsLastModified(): string
    {
        return Option::get(self::getModuleId(), 'GEO_IDS_MODIFIED', '');
    }

    public function getFioPropType(): string
    {
        return $this->fioPropType;
    }

    public function setFioPropType(string $fioPropType): void
    {
        $this->fioPropType = $fioPropType;
    }

    public function getFirstname(): string
    {
        return $this->firstname;
    }

    public function setFirstname(string $firstname): void
    {
        $this->firstname = $firstname;
    }

    public function getSecondname(): string
    {
        return $this->secondname;
    }

    public function setSecondname(string $secondname): void
    {
        $this->secondname = $secondname;
    }

    public function getMiddlename(): string
    {
        return $this->middlename;
    }

    public function setMiddlename(string $middlename): void
    {
        $this->middlename = $middlename;
    }

    public function getAddressPropType(): string
    {
        return $this->addressPropType;
    }

    public function setAddressPropType(string $addressPropType): void
    {
        $this->addressPropType = $addressPropType;
    }

    public function getStreet(): string
    {
        return $this->street;
    }

    public function setStreet(string $street): void
    {
        $this->street = $street;
    }

    public function getHouse(): string
    {
        return $this->house;
    }

    public function setHouse(string $house): void
    {
        $this->house = $house;
    }

    public function getFlat(): string
    {
        return $this->flat;
    }

    public function setFlat(string $flat): void
    {
        $this->flat = $flat;
    }

    public function isFioOneProp(): bool
    {
        return $this->fioPropType === '1';
    }

    public function isAddressOneProp(): bool
    {
        return $this->addressPropType === '1';
    }

    public function getForcePaymentType(): string
    {
        return $this->forcePaymentType;
    }

    public function setForcePaymentType(string $forcePaymentType): void
    {
        $this->forcePaymentType = $forcePaymentType;
    }

    public function isForcePaymentTypeSet(): bool
    {
        return $this->forcePaymentType !== '';
    }

    public function getForcePaySum($paySum): float
    {
        return $this->forcePaymentType === '2' ? 0 : $paySum;
    }

    public function getParselcreateForcePaymentType(): string
    {
        return $this->parselcreateForcePaymentType;
    }

    public function setParselcreateForcePaymentType(string $parselcreateForcePaymentType): void
    {
        $this->parselcreateForcePaymentType = $parselcreateForcePaymentType;
    }

    public function isParselcreateForcePaymentTypeSet(): bool
    {
        return $this->parselcreateForcePaymentType !== '';
    }

    public function getParselcreateForcePaySum($paySum): float
    {
        return $this->parselcreateForcePaymentType === '2' ? 0 : $paySum;
    }

}