<?php

namespace Boxberry\Bitrix\Helpers;

use Bitrix\Main\Application;
use Bitrix\Main\ArgumentException;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Result;
use Bitrix\Main\Session\SessionInterface;
use Bitrix\Main\Web\Json;
use Bitrix\Main\Page\Asset;
use Bitrix\Main\Context;

class Widget
{
    private const NAME = 'boxberry_widget';
    private SessionInterface $session;

    public function __construct()
    {
        $this->session = Application::getInstance()->getSession();
        $this->session->start();
    }

    public function getDivElement(): string
    {
        return '<div id="' . $this->getDivId() . '">' . $this->getButtonHtml() . '</div>';
    }

    public function getButtonHtml(): string
    {
        if ($this->isPvzIdSet() && $this->isPvzAddressSet()) {
            return '<button class="btn btn-default btn-primary boxberry_widget" href="#">' . Loc::getMessage('SELECT_ANOTHER_PVZ') . '</button><p>' . Loc::getMessage('SELECTED_PVZ') . $this->getPvzAddress() . '</p>';
        }

        return '<button class="btn btn-default btn-primary boxberry_widget" href="#">' . Loc::getMessage('SELECT_PVZ') . '</button><p></p>';
    }

    public function removeByName($names): void
    {
        if (is_string($names)) {
            $this->session->remove('bb_' . $names);
        }

        if (is_array($names)) {
            foreach ($names as $name) {
                $this->session->remove('bb_' . $name);
            }
        }
    }

    public function clearAll(): void
    {
        $paramsNames = [
            'widgetKey',
            'customCity',
            'targetStart',
            'ordersum',
            'weight',
            'paysum',
            'height',
            'width',
            'depth',
            'addressPropId',
            'address',
            'pvzId',
            'pvzAddress',
            'pvzProfileIds',
            'cityCode',
            'divId',
            'addToDescription',
            'consoleLogEnabled',
            'addAfterDescription',
            'addressPvzReadOnly',
            'addToCustomId',
            'customId',
            'isAddressOneProp',
            'cityName',
            'street',
            'house',
            'flat',
            'streetPropId',
            'housePropId',
            'flatPropId',
        ];

        $this->removeByName($paramsNames);
    }

    public function getWidgetParams(): Result
    {
        $result = new Result();

        $result->setData([
            'widgetKey' => $this->getWidgetKey(),
            'customCity' => $this->getCustomCity(),
            'targetStart' => $this->getTargetStart(),
            'ordersum' => $this->getOrderSum(),
            'weight' => $this->getWeight(),
            'paysum' => $this->getPaySum(),
            'height' => $this->getHeight(),
            'width' => $this->getWidth(),
            'depth' => $this->getDepth(),
            'addressPropId' => $this->getAddressPropId(),
            'address' => $this->getPvzAddress(),
            'pvzId' => $this->getPvzId(),
            'pvzProfileIds' => $this->getPvzProfileIds(),
            'addToDescription' => $this->getAddToDescription(),
            'currentId' => $this->getCurrentId(),
            'consoleLogEnabled' => $this->getConsoleLogEnabled(),
            'addAfterDescription' => $this->getAddAfterDescription(),
            'addressPvzReadOnly' => $this->getAddressPvzReadOnly(),
            'addToCustomId' => $this->getAddToCustomId(),
            'customId' => $this->getCustomId(),
            'isAddressOneProp' => $this->getIsAddressOneProp(),
            'cityName' => $this->getCityName(),
            'street' => $this->getStreet(),
            'house' => $this->getHouse(),
            'flat' => $this->getFlat(),
            'streetPropId' => $this->getStreetPropId(),
            'housePropId' => $this->getHousePropId(),
            'flatPropId' => $this->getFlatPropId(),
        ]);

        return $result;

    }

    public function getStreetPropId()
    {
        return $this->session->get('bb_streetPropId');
    }

    public function setStreetPropId($streetPropId): void
    {
        $this->set('streetPropId', $streetPropId);
    }

    public function getHousePropId()
    {
        return $this->session->get('bb_housePropId');
    }

    public function setHousePropId($housePropId): void
    {
        $this->set('housePropId', $housePropId);
    }

    public function getFlatPropId()
    {
        return $this->session->get('bb_flatPropId');
    }

    public function setFlatPropId($flatPropId): void
    {
        $this->set('flatPropId', $flatPropId);
    }

    public function setCityName($cityName): void
    {
        $this->set('cityName', $cityName);
    }

    public function getCityName()
    {
        return $this->session->get('bb_cityName');
    }

    public function getStreet()
    {
        return $this->session->get('bb_street');
    }

    public function setStreet($street): void
    {
        $this->set('street', $street);
    }

    public function getHouse()
    {
        return $this->session->get('bb_house');
    }

    public function setHouse($house): void
    {
        $this->set('house', $house);
    }

    public function getFlat()
    {
        return $this->session->get('bb_flat');
    }

    public function setFlat($flat): void
    {
        $this->set('flat', $flat);
    }

    public function getIsAddressOneProp()
    {
        return $this->session->get('bb_isAddressOneProp');
    }

    public function setIsAddressOneProp($isAddressOneProp): void
    {
        $this->set('isAddressOneProp', $isAddressOneProp);
    }

    public function getAddToCustomId()
    {
        return $this->session->get('bb_addToCustomId');
    }

    public function setAddToCustomId($addToCustomId): void
    {
        $this->set('addToCustomId', $addToCustomId);
    }

    public function getCustomId()
    {
        return $this->session->get('bb_customId');
    }

    public function setCustomId($customId): void
    {
        $this->set('customId', $customId);
    }

    public function getAddressPvzReadOnly()
    {
        return $this->session->get('bb_addressPvzReadOnly');
    }

    public function setAddressPvzReadOnly($addressPvzReadOnly): void
    {
        $this->set('addressPvzReadOnly', $addressPvzReadOnly);
    }

    public function getAddAfterDescription()
    {
        return $this->session->get('bb_addAfterDescription');
    }

    public function setAddAfterDescription($addAfterDescription)
    {
        $this->set('addAfterDescription', $addAfterDescription);
    }

    public function getAddToDescription(): ?string
    {
        return $this->session->get('bb_addToDescription');
    }

    public function setAddToDescription($addToDescription): void
    {
        $this->set('addToDescription', $addToDescription);
    }

    public function getConsoleLogEnabled()
    {
        return $this->session->get('bb_consoleLogEnabled');
    }

    public function setConsoleLogEnabled($consoleLogEnabled)
    {
        $this->set('consoleLogEnabled', $consoleLogEnabled);
    }

    public function isParamSet($name): bool
    {
       return $this->session->has($name) && !empty($this->session->get($name));
    }

    public function getName(): string
    {
        return self::NAME;
    }

    /**
     * @throws ArgumentException
     */
    private function set($name, $value): void
    {
        $this->session->set('bb_' . $name, $value);
    }

    public function setWidgetKey($token): void
    {
        $this->set('widgetKey', $token);
    }

    public function getWidgetKey(): ?string
    {
        return $this->session->get('bb_widgetKey');
    }

    public function setCustomCity($city): void
    {
        $this->set('customCity', $city);
    }

    public function getCustomCity(): ?string
    {
        return $this->session->get('bb_customCity');
    }

    public function setTargetStart($targetStart): void
    {
        $this->set('targetStart', $targetStart);
    }

    public function getTargetStart(): ?string
    {
        return $this->session->get('bb_targetStart');
    }

    public function setOrderSum($orderSum): void
    {
        $this->set('ordersum', $orderSum);
    }

    public function getOrderSum(): int|float|null
    {
        return $this->session->get('bb_ordersum');
    }

    public function setWeight($weight): void
    {
        $this->set('weight', $weight);
    }

    public function getWeight(): int|float|null
    {
        return $this->session->get('bb_weight');
    }

    public function setPaySum($paySum): void
    {
        $this->set('paysum', $paySum);
    }

    public function getPaySum(): int|float|null
    {
        return $this->session->get('bb_paysum');
    }

    public function setHeight($height): void
    {
        $this->set('height', $height);
    }

    public function getHeight(): int|float|null
    {
        return $this->session->get('bb_height');
    }

    public function setWidth($width): void
    {
        $this->set('width', $width);
    }

    public function getWidth(): int|float|null
    {
        return $this->session->get('bb_width');
    }

    public function setDepth($depth): void
    {
        $this->set('depth', $depth);
    }

    public function getDepth(): int|float|null
    {
        return $this->session->get('bb_depth');
    }

    public function setAddressPropId($addressPropId): void
    {
        $this->set('addressPropId', $addressPropId);
    }

    public function getAddressPropId(): ?string
    {
        return $this->session->get('bb_addressPropId');
    }

    public function setAddress($address): void
    {
        $this->set('address', $address);
    }

    public function getAddress(): ?string
    {
        return $this->session->get('bb_address');
    }

    public function setPvzProfileIds($pvzProfileIds = []): void
    {
        $this->set('pvzProfileIds', $pvzProfileIds);
    }

    public function getPvzProfileIds(): ?array
    {
        return $this->session->get('bb_pvzProfileIds');
    }

    public function setCityCode($cityCode): void
    {
        $this->set('cityCode', $cityCode);
    }

    public function getCityCode(): ?string
    {
        return $this->session->get('bb_cityCode');
    }

    public function setPvzId($pvzId): void
    {
        $this->set('pvzId', $pvzId);
    }

    public function getPvzId(): ?string
    {
        return $this->session->get('bb_pvzId');
    }

    public function setPvzAddress($pvzAddress): void
    {
        $this->set('pvzAddress', $pvzAddress);
    }

    public function getPvzAddress(): ?string
    {
        return $this->session->get('bb_pvzAddress');
    }

    public function setDivId($divId): void
    {
        $this->set('divId', $divId);
    }

    public function getDivId(): ?string
    {
        return $this->session->get('bb_divId');
    }

    public function isSet($name): bool
    {
        return $this->session->has('bb_' . $name) && !empty($this->session->get('bb_' . $name));
    }

    public function isPvzIdSet(): bool
    {
        return $this->isSet('pvzId');
    }

    public function isPvzAddressSet(): bool
    {
        return $this->isSet('pvzAddress');
    }

    public function isCityCodeSet(): bool
    {
        return $this->isSet('cityCode');
    }

    public function removePvzAndAddress(): void
    {
        $this->removeByName(['pvzId', 'pvzAddress']);
    }

    public function getCurrentId(): ?string
    {
        return $this->session->get('bb_currentId');
    }

    public function setCurrentId($id): void
    {
        $this->set('currentId', $id);
    }


}