<?php

namespace Boxberry\Delivery\Controllers;

use Bitrix\Main\Engine\Controller;
use Bitrix\Main\Engine\Response\AjaxJson;
use Bitrix\Main\Error;
use Bitrix\Main\ErrorCollection;
use Bitrix\Main\Localization\Loc;
use Boxberry\Api\Client;
use Boxberry\Bitrix\Helpers\Delivery;
use Boxberry\Bitrix\PointsForParcelsTable;
use Bitrix\Main\Engine\ActionFilter;
use Boxberry\Delivery\ActionFilter\Referer;

Loc::loadMessages(__FILE__);

class Service extends Controller
{
    public function getPointsForParcelsAction($searchTerm): AjaxJson
    {
        $result = PointsForParcelsTable::getPointsForParcels($searchTerm);

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function clearCacheAction($type): AjaxJson
    {
        $result = Delivery::clearCache($type);

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function updateDeliveryDataAction($apiToken = '', $updateDataType = 'all'): AjaxJson
    {
        $result = Delivery::updateDeliveryData($apiToken, $updateDataType);

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function getEmptyDeliveryDataAction(): AjaxJson
    {
        $result = Delivery::getEmptyDeliveryData();

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function validateApiTokenAction($apiToken): AjaxJson
    {
        $result = (new Client($apiToken))->checkApiToken();

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function validatePointForParcelAction($pointForParcel): AjaxJson
    {
        $result = PointsForParcelsTable::getPointCodeByName($pointForParcel);

        if ($result) {
            return AjaxJson::createSuccess(['code' => $result]);
        } else {
            return AjaxJson::createError(new ErrorCollection([new Error(Loc::getMessage('POINT_FOR_PARCEL_WITH_THIS_NAME_NOT_FOUND_CHECK_NAME'))]));
        }
    }

    protected function getDefaultPreFilters(): array
    {

        return array_merge(
            parent::getDefaultPreFilters(),
            [
                new ActionFilter\Cors(),
                new ActionFilter\Scope(ActionFilter\Scope::AJAX),
                new Referer([
                    Referer::SALE_ORDER_VIEW_REFERER,
                    Referer::SALE_DELIVERY_SERVICE_EDIT_REFERER
                ])
            ]
        );
    }
}