<?php

namespace Boxberry\Delivery\Controllers;

use Bitrix\Main\Engine\Controller;
use Bitrix\Main\Engine\ActionFilter;
use Bitrix\Main\Engine\Response\AjaxJson;
use Bitrix\Main\Error;
use Bitrix\Main\ErrorCollection;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Result;
use Boxberry\Api\Client;
use Boxberry\Bitrix\Helpers\Widget as WidgetHelper;

Loc::loadMessages(__FILE__);

class Widget extends Controller
{
    public function getAction(): AjaxJson
    {
        $result = (new WidgetHelper())->getWidgetParams();

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($result->getData());
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    public function setPickupPointAction($id, $address): AjaxJson
    {
        $result = new Result();
        $params = new WidgetHelper();
        $client = new Client(substr($params->getWidgetKey(), 2));
        $data = [];

        try {
            $pointsDescription = $client->pointsDescription($id);

            if ($pointsDescription->getCityCode() !== $params->getCityCode()) {
                $result->addError(new Error(Loc::getMessage('FOR_ORDER_PVZ_CITY_CODE_IS_NOT_MATCH'), 0, ['checkLocation' => false]));
                return AjaxJson::createError(new ErrorCollection($result->getErrors()));
            }

            $params->setCityName($pointsDescription->getCityName());
            $params->setStreet($pointsDescription->getStreet());
            $params->setHouse($pointsDescription->getHouse());
            $params->setFlat($pointsDescription->getApartment());

            $data = array_merge($data, [
                'city' => $pointsDescription->getCityName(),
                'street' => $pointsDescription->getStreet(),
                'house' => $pointsDescription->getHouse(),
                'flat' => $pointsDescription->getApartment()
            ]);

            $params->setPvzId($id);
            $params->setPvzAddress($address);

            $data = array_merge($data, [
                'message' => Loc::getMessage('FOR_ORDER_PVZ_CODE_IS_SET') . $id . Loc::getMessage('ADDRESS') . $address,
                'checkLocation' => true,
            ]);

        } catch (\Exception $e) {
            $result->addError(new Error($e->getMessage()));
        }

        if ($result->isSuccess()) {
            return AjaxJson::createSuccess($data);
        } else {
            return AjaxJson::createError(new ErrorCollection($result->getErrors()));
        }
    }

    protected function getDefaultPreFilters(): array
    {
        return [
            new ActionFilter\HttpMethod(
                [
                    ActionFilter\HttpMethod::METHOD_GET,
                    ActionFilter\HttpMethod::METHOD_POST
                ]
            ),
            new ActionFilter\Csrf(),
            new ActionFilter\Cors(),
            new ActionFilter\Scope(ActionFilter\Scope::AJAX)
        ];
    }

}